Attribute VB_Name = "modGTFEPG"
Option Explicit

Function TURBP(T1 As Single, P1 As Single, PR As Single, e_poly As Single, n As Integer, Comp As Variant, _
iNumComp As Integer, iMode As Integer) As Variant
'This function calculates expander power (per unit mass flow rate) and discharge properties (pressure, temperature,
'enthalpy and entropy) using a user-provided polytropic efficiency, expander pressure ratio and gas composition and
' suction properties (pressure and temperature).  The function uses JANAF properties.
'
' T1: Turbine inlet temperature
' P1: Turbine inlet pressure
' PR: Turbine pressure ratio
' e_poly: polytropic efficiency
' N: Number of elementary stages
' Comp: Gas composition
' iNumComp: Fas composition array size
' iMode: Units
'
Dim PR_stage As Single
Dim Deltah_s As Single, Deltah As Single
Dim w_turb As Single
Dim P2 As Single, T2 As Single, h2 As Single, s2 As Single
Dim T2s As Single, h2s As Single, s1 As Single
Dim i As Integer

ReDim hi1(1 To n) As Single
ReDim hi2(1 To n) As Single
ReDim hi2s(1 To n) As Single
ReDim Ti1(1 To n) As Single
ReDim Ti2(1 To n) As Single
ReDim Ti2s(1 To n) As Single
ReDim si1(1 To n) As Single
ReDim si2(1 To n) As Single
ReDim Pi1(1 To n) As Single
ReDim Pi2(1 To n) As Single
Dim vOutputArray(1 To 7, 1) As Single
On Local Error GoTo err_here

' Elementary stage pressure ratio
PR_stage = PR ^ (1 / n)
' Inlet properties
hi1(1) = props("h", T1, P1, Comp, iMode)
Ti1(1) = T1
Pi1(1) = P1
si1(1) = props("s", T1, P1, Comp, iMode)
s1 = si1(1)

' Stage-by-stage calculation
For i = 1 To n
    ' Stage exit pressure
    Pi2(i) = Pi1(i) / PR_stage
    ' Stage exit isentropic temperature
    Ti2s(i) = TSP(Ti1(i) + 1, 2 * Ti1(i), Comp, iNumComp, iMode, Pi2(i), si1(i))
    ' Stage exit isentropic enthalpy
    hi2s(i) = props("h", Ti2s(i), Pi2(i), Comp, iMode)
    Deltah_s = hi1(i) - hi2s(i)
    Deltah = Deltah_s * e_poly
    ' Stage exit actual enthalpy
    hi2(i) = hi1(i) - Deltah
    ' Stage exit actual temperature
    Ti2(i) = TH(Ti1(i) + 1, 2 * Ti1(i), Comp, iNumComp, iMode, hi2(i))
    ' Stage exit actual entropy
    si2(i) = props("s", Ti2(i), Pi2(i), Comp, iMode)
    ' Next stage inlet properties
    If i < n Then
        hi1(i + 1) = hi2(i)
        Ti1(i + 1) = Ti2(i)
        Pi1(i + 1) = Pi2(i)
        si1(i + 1) = si2(i)
    End If
Next i

' Compressor exit
h2 = hi2(n)
s2 = si2(n)
T2 = Ti2(n)
P2 = Pi2(n)
w_turb = h2 - hi1(1)
T2s = TSP(T1 + 10, 900, Comp, iNumComp, iMode, P2, s1)
h2s = props("h", T2s, P2, Comp, iMode)

' Outputs
vOutputArray(1, 0) = P2
vOutputArray(2, 0) = T2
vOutputArray(3, 0) = h2
vOutputArray(4, 0) = s2
vOutputArray(5, 0) = w_turb
vOutputArray(6, 0) = T2s
vOutputArray(7, 0) = h2s
TURBP = vOutputArray
Exit Function

err_here:
    TURBP = Err.Number
End Function

Function COMPP(T1 As Single, P1 As Single, PR As Single, e_poly As Single, n As Integer, Comp As Variant, _
iNumComp As Integer, iMode As Integer) As Variant
' This function calculates compressor power (per unit mass flow rate) and discharge properties (pressure, temperature,
' enthalpy and entropy)
' using a user-provided polytropic efficiency, compressor pressure ratio and gas composition and suction properties
' (pressure and temperature).  The function uses JANAF properties.
'
' T1: Compressor inlet temperature
' P1: Compressor inlet pressure
' PR: Compressor pressure ratio
' e_poly: polytropic efficiency
' N: Number of elementary stages
' Comp: Gas composition
' iNumComp: Fas composition array size
' iMode: Units
'
Dim PR_stage As Single
Dim Deltah_s As Single, Deltah As Single
Dim w_comp As Single
Dim P2 As Single, T2 As Single, h2 As Single, s2 As Single
Dim T2s As Single, h2s As Single, s1 As Single
Dim i As Integer

ReDim hi1(1 To n) As Single
ReDim hi2(1 To n) As Single
ReDim hi2s(1 To n) As Single
ReDim Ti1(1 To n) As Single
ReDim Ti2(1 To n) As Single
ReDim Ti2s(1 To n) As Single
ReDim si1(1 To n) As Single
ReDim si2(1 To n) As Single
ReDim Pi1(1 To n) As Single
ReDim Pi2(1 To n) As Single
Dim vOutputArray(1 To 7, 1) As Single
On Local Error GoTo err_here

' Elementary stage pressure ratio
PR_stage = PR ^ (1 / n)
' Inlet properties
hi1(1) = props("h", T1, P1, Comp, iMode)
Ti1(1) = T1
Pi1(1) = P1
si1(1) = props("s", T1, P1, Comp, iMode)
s1 = si1(1)

' Stage-by-stage calculation
For i = 1 To n
    ' Stage exit pressure
    Pi2(i) = PR_stage * Pi1(i)
    ' Stage exit isentropic temperature
    Ti2s(i) = TSP(Ti1(i) + 1, 2 * Ti1(i), Comp, iNumComp, iMode, Pi2(i), si1(i))
    ' Stage exit isentropic enthalpy
    hi2s(i) = props("h", Ti2s(i), Pi2(i), Comp, iMode)
    Deltah_s = hi2s(i) - hi1(i)
    Deltah = Deltah_s / e_poly
    ' Stage exit actual enthalpy
    hi2(i) = hi1(i) + Deltah
    ' Stage exit actual temperature
    Ti2(i) = TH(Ti1(i) + 1, 2 * Ti1(i), Comp, iNumComp, iMode, hi2(i))
    ' Stage exit actual entropy
    si2(i) = props("s", Ti2(i), Pi2(i), Comp, iMode)
    ' Next stage inlet properties
    If i < n Then
        hi1(i + 1) = hi2(i)
        Ti1(i + 1) = Ti2(i)
        Pi1(i + 1) = Pi2(i)
        si1(i + 1) = si2(i)
    End If
Next i

' Compressor exit
h2 = hi2(n)
s2 = si2(n)
T2 = Ti2(n)
P2 = Pi2(n)
w_comp = h2 - hi1(1)
T2s = TSP(T1 + 10, 900, Comp, iNumComp, iMode, P2, s1)
h2s = props("h", T2s, P2, Comp, iMode)

' Outputs
vOutputArray(1, 0) = P2
vOutputArray(2, 0) = T2
vOutputArray(3, 0) = h2
vOutputArray(4, 0) = s2
vOutputArray(5, 0) = w_comp
vOutputArray(6, 0) = T2s
vOutputArray(7, 0) = h2s
COMPP = vOutputArray
Exit Function

err_here:
    COMPP = Err.Number
End Function

Function GTURBC(PRc As Single, T39 As Single, TF As Single, m1 As Single, vInputArray As Variant) As Variant
' CHAPTER 8
' Simple air-cooled turbine model
' S. C. Gulen
'
' This function represents a simple gas turbine model with "pseudo" compressor and cooled turbine stages.
' The methodology is described in Chapter 8.
'
' See Table C.3.1 for the input array
'
' See Table C.3.2 for the output array
'
Dim g1 As Single, g2 As Single, k As Single
Dim Cpa As Single, Cpg As Single, Cpf As Single
Dim dP_in As Single, dP_ex As Single, dP_cmb As Single
Dim eta_c As Single, eta_t As Single, eta_m As Single, eta_cmb As Single, eta_gen As Single
Dim c_f As Single

Dim PRt As Single, PRt_1 As Single, PRt_2 As Single, PRt_3 As Single, PRt_4 As Single
Dim PRc_1 As Single, PRc_2 As Single, PRc_3 As Single
Dim PCD As Single, TCD As Single, P39 As Single, T40 As Single
Dim Pt1_ex As Single, Pt2_ex As Single, Pt3_ex As Single
Dim Tt1_ex As Single, Tt2_ex As Single, Tt3_ex As Single, Tt4_ex As Single
Dim Tt2_in As Single, Tt3_in As Single, Tt4_in As Single
Dim mt2_in As Single, mt3_in As Single, mt4_in As Single
Dim T_a As Single, T_b As Single, T_c As Single, T_d As Single
Dim T_bm1 As Single, T_bm2 As Single, T_bm3 As Single
Dim Trr_31 As Single, Trr_32 As Single

Dim nch As Single, ch1 As Single, ch2 As Single
Dim nch_new As Single, ch1_new As Single, ch2_new As Single
Dim m_nch As Single, m_ch1 As Single, m_ch2 As Single
Dim m20 As Single, m_39 As Single, m_40 As Single, m_f As Single

Dim b_1 As Single, b_2 As Single, b_3 As Single
Dim k_nch As Single, k_ch1 As Single, k_ch2 As Single
Dim fi As Single, fi_inf1 As Single, fi_inf2 As Single, fi_inf3 As Single
Dim tau1 As Single, tau2 As Single, tau3 As Single

Dim DEN As Single, h1 As Single, h2 As Single, h3 As Single, QIN As Single, HBERR As Single

Dim Wc1 As Single, Wc2 As Single, Wc3 As Single, Wc_tot As Single
Dim Wt1 As Single, Wt2 As Single, Wt3 As Single, Wt4 As Single, Wt_tot As Single
Dim Wnet As Single
Dim Texh As Single, mexh As Single
Dim Eff As Single
Dim f_to_a As Single
Dim eps As Single

Dim vOutputArray(1 To 30, 1) As Single

Const N_stgs As Integer = 4
Const Tref As Single = 77
Const LHV As Single = 21515
Const P1 As Single = 14.7
Const T1 As Single = 59
Const Runv As Single = 1.986
eps = vInputArray(10)

If vInputArray(11) < 0 Then
    T40 = 0.7744 * T39 + 438.32
    c_f = 1.258 * (T40 / 1000) ^ -0.385
Else
    c_f = vInputArray(11)
End If

On Local Error GoTo err_here
'
Cpa = vInputArray(1)
g1 = Cpa / (Cpa - Runv / 28.97)
If vInputArray(2) < 0 Then
    T40 = 0.7744 * T39 + 438.32
    Cpg = 0.2243 + 0.0316 * (T40 / 1000)    'across the turbine
Else
    Cpg = vInputArray(2)
End If
g2 = Cpg / (Cpg - Runv / 28.97)
Cpf = 0.65  'for book figures!!!
eta_c = vInputArray(3)
eta_t = vInputArray(4)
eta_cmb = 0.995
eta_m = 0.995
eta_gen = 0.989
' Initial guess:
nch = 0.09
ch1 = 0.08
ch2 = 0.07
'Tf = 365
dP_cmb = 0.05
dP_in = 0.0075
dP_ex = 0.015
' technology numbers
tau1 = vInputArray(12)
tau2 = vInputArray(13)
tau3 = vInputArray(14)
' from Muenzberg and Kurzke
k_nch = vInputArray(15) '0.0533
k_ch1 = vInputArray(16) 'k_nch * 2
k_ch2 = vInputArray(17) 'k_nch
' Maximum cooling effectiveness
fi_inf1 = 1
fi_inf2 = fi_inf1
fi_inf3 = fi_inf1
' Exponent beta
b_1 = 1
b_2 = b_1
b_3 = b_1
' Allowable metal temperatures
T_bm1 = vInputArray(5)
T_bm2 = vInputArray(6)
T_bm3 = vInputArray(7)
'Turbine PR:
PRt = PRc * (1 - dP_cmb) * (1 - dP_in) * (1 - dP_ex)
PRt = PRt * 0.95
PRt_1 = PRt ^ (1 / N_stgs)
PRt_2 = PRt_1
PRt_3 = PRt_2
PRt_4 = PRt / (PRt_1 * PRt_2 * PRt_3)

'Compressor discharge pressure:
PCD = PRc * P1
' Turbine inlet pressure
P39 = PCD * (1 - dP_cmb)
' Turbine stages 1 and 2 exit pressures:
Pt1_ex = P39 / PRt_1
Pt2_ex = Pt1_ex / PRt_2
' Compressor stage exit pressures:
PRc_1 = (Pt2_ex + 15) / (P1 * (1 - dP_in))
PRc_2 = (Pt1_ex + 15) / (P1 * (1 - dP_in) * PRc_1)
PRc_3 = PRc / (PRc_1 * PRc_2)
' Compressor stage exit temperatures:
k = 1 - 1 / g1
T_a = (T1 + 459.67) * PRc_1 ^ (k / eta_c) - 459.67
T_b = (T_a + 459.67) * PRc_2 ^ (k / eta_c) - 459.67
T_c = (T_b + 459.67) * PRc_3 ^ (k / eta_c) - 459.67
T_d = T_c
TCD = T_d
'
10:
m_nch = nch * m1
m_ch1 = ch1 * m1
m_ch2 = ch2 * m1
m20 = m1 - m_nch - m_ch1 - m_ch2
' Combustor:
DEN = LHV * eta_cmb + Cpf * (TF - Tref)
h2 = Cpg * (T39 + 459.67)
h1 = Cpa * (TCD + 459.67)
m_f = m20 * (h2 - h1) / (DEN - h2)
f_to_a = m_f / m1
m_39 = m20 + m_f
'
fi = (T39 - T_bm1) / (T39 - T_c)
nch_new = tau1 * k_nch * (m_39 / m1) * (1 / (fi_inf1 / fi - 1)) ^ b_1

If Abs(nch_new / nch - 1) > eps Then
    nch = nch_new
    GoTo 10
End If
nch = nch_new
'
m_40 = m_39 + m_nch
' Firing temperature:
T40 = (m_nch * Cpa * (T_c + 459.67) + m_39 * Cpg * (T39 + 459.67)) / (m_40 * Cpg) - 459.67

'do NOT use this for book figures!!!
'eta_t = 0.965 - 0.01 * Log(PRt) * (1 + vInputArray(9) * Sqr(nch + ch1 + ch2))

' 1st stage exit:
k = 1 - 1 / g2
Tt1_ex = (T40 + 459.67) / PRt_1 ^ (eta_t * k) - 459.67
' 2nd stage inlet:
mt2_in = m_40 + m_ch1
Tt2_in = (m_ch1 * Cpa * (T_b + 459.67) + m_40 * Cpg * (Tt1_ex + 459.67)) / (mt2_in * Cpg) - 459.67
' 2nd stage exit:
Tt2_ex = (Tt2_in + 459.67) / PRt_2 ^ (eta_t * k) - 459.67

Trr_31 = Tt2_ex

fi = (Trr_31 - T_bm2) / (Trr_31 - T_b)
ch1_new = tau2 * k_ch1 * (mt2_in / m1) * (1 / (fi_inf2 / fi - 1)) ^ b_2

If Abs(ch1_new / ch1 - 1) > eps Then
    ch1 = ch1_new
    GoTo 10
End If
ch1 = ch1_new

' 3rd stage inlet:
mt3_in = mt2_in + m_ch2
Tt3_in = (m_ch2 * Cpa * (T_a + 459.67) + mt2_in * Cpg * (Tt2_ex + 459.67)) / (mt3_in * Cpg) - 459.67

' 3rd stage exit:
Tt3_ex = (Tt3_in + 459.67) / PRt_3 ^ (eta_t * k) - 459.67

Trr_32 = Tt3_ex

fi = (Trr_32 - T_bm3) / (Trr_32 - T_a)
If (fi < 0) Then
    ch2_new = 0
Else
    ch2_new = tau3 * k_ch2 * (mt3_in / m1) * (1 / (fi_inf3 / fi - 1)) ^ b_3
End If

If Abs(ch2_new / ch2 - 1) > eps Then
    ch2 = ch2_new
    GoTo 10
End If
ch2 = ch2_new

20:
' 4th stage (uncooled) inlet:
mt4_in = mt3_in
Tt4_in = Tt3_ex

' 4th stage exit:
Tt4_ex = (Tt4_in + 459.67) / PRt_4 ^ (eta_t * k) - 459.67

' Compressor Work:
k = 1 - 1 / g1
Wc1 = m1 * Cpa * (T1 + 459.67) * (PRc_1 ^ (k / eta_c) - 1)
Wc2 = (m1 - m_ch2) * Cpa * (T_a + 459.67) * (PRc_2 ^ (k / eta_c) - 1)
Wc3 = (m1 - m_ch2 - m_ch1) * Cpa * (T_b + 459.67) * (PRc_3 ^ (k / eta_c) - 1)
Wc_tot = (Wc1 + Wc2 + Wc3) * 1.05506

' Turbine Work
Wt1 = m_40 * Cpg * (T40 - Tt1_ex)
Wt2 = mt2_in * Cpg * (Tt2_in - Tt2_ex)
Wt3 = mt3_in * Cpg * (Tt3_in - Tt3_ex)
Wt4 = mt4_in * Cpg * (Tt4_in - Tt4_ex)

' Shaft Work:
Wt_tot = (Wt1 + Wt2 + Wt3 + Wt4) * 1.05506

Wnet = (Wt_tot - Wc_tot) * eta_m    'Mechanical losses
Wnet = Wnet * eta_gen   'Generator losses
Texh = Tt4_ex
mexh = m1 + m_f

' Heat balance error check:
QIN = m_f * (LHV + Cpf * (TF - Tref))
h1 = Cpa * m1 * (T1 + 459.67)
h2 = Cpg * mexh * (Texh + 459.67)
HBERR = h1 + QIN - h2 - (Wt_tot - Wc_tot) / 1.05506
HBERR = HBERR - m_f * (1 - eta_cmb) * LHV
HBERR = HBERR / QIN
Eff = Wnet / (m_f * LHV * 1.05506)

' Efficiency calibration
Eff = Eff / c_f
f_to_a = f_to_a * c_f

vOutputArray(1, 0) = Wnet
vOutputArray(2, 0) = Eff
vOutputArray(3, 0) = f_to_a
vOutputArray(4, 0) = nch
vOutputArray(5, 0) = ch1 + ch2
vOutputArray(6, 0) = T40
vOutputArray(7, 0) = Texh
vOutputArray(8, 0) = mexh
vOutputArray(9, 0) = T_a
vOutputArray(10, 0) = T_b
vOutputArray(11, 0) = T_c
vOutputArray(12, 0) = HBERR

GTURBC = vOutputArray

Exit Function
err_here:
GTURBC = Err.Number
'Err.Description

End Function

Function GTURBUC(PRc As Single, T3 As Single, TF As Single, vInputArray As Variant, iOutput As Integer) As Variant
'
' CHAPTER 8
' Simple uncooled turbine model
' S. C. Gulen
'
' This function encapsulates Equations 8.1 and 8.6 to model a gas turbine without explicit HGP cooling calculations.
' The methodology is described in Chapter 8.
'
Dim g1 As Single, g2 As Single, ka As Single, kg As Single
Dim Cpa As Single, Cpg As Single, Cpf As Single
Dim PR_S1N As Single, dP_cmb As Single
Dim eta_c As Single, eta_t As Single, eta_m As Single, eta_cmb As Single, eta_gen As Single
Dim f As Single, c_f As Single, HB_Err As Single

Dim PRt As Single, w_comp As Single, w_turb As Single
Dim tau3 As Single, ksi_g As Single, ksi_f As Single
Dim T2 As Single, T4 As Single, T4_Corr As Single

Dim DEN As Single, NUMER As Single, ll As Single
' INPUTS
' PRc = Compressor pressure ratio
' T3 = Firing Temperature in F
' TF = Fuel temperature in F
' vInputArray(1) = Air specific heat, 0.25 Btu/lb-R
' vInputArray(2) = Gas specific heat, 1 or 2 - see the code below
' vInputArray(3) = Compressor polytropic efficiency as a fraction
' vInputArray(4) = Turbine polytropic efficiency as a fraction
' vInputArray(5) = pressure ratio across S1N
' vInputArray(6) = c_f (if a negative value, it is calculated below in the code
' vInputArray(6) = Exhaust temperature correction in F
' Dim vOutputArray(1 To 30, 1) As Single
'
' OUTPUTS
' - if iOutput is 1 --> GT efficiency
' - if iOutput is 2 --> GT output
' - if iOutput is 3 --> GT exhaust temperature
' - if iOutput is 4 --> Heat balance error as %
'
Const Tref As Single = -459.67
Const LHV As Single = 21515
Const P1 As Single = 14.7
Const T1 As Single = 59
Const Runv As Single = 1.986

On Local Error GoTo err_here
'
If vInputArray(6) < 0 Then
    c_f = 1.216 * (T3 / 1000) ^ -0.385
Else
    c_f = vInputArray(6)    '0.813
End If
T4_Corr = vInputArray(7)
' Average air cp
Cpa = vInputArray(1)
g1 = Cpa / (Cpa - Runv / 28.97)
ka = 1 - 1 / g1
' Average gas cp
If vInputArray(2) = 1 Then
    Cpg = 0.244 + 0.0291 * (T3 / 1000) '@ turbine inlet
Else
    Cpg = 0.2243 + 0.0316 * (T3 / 1000)    'across the turbine
End If
ksi_g = Cpg / Cpa
g2 = Cpg / (Cpg - Runv / 28.97)
kg = 1 - 1 / g2
Cpf = 0.6
ksi_f = Cpf / Cpa
eta_c = vInputArray(3)
eta_t = vInputArray(4)
eta_cmb = 0.995
eta_m = 0.995
eta_gen = 0.989
dP_cmb = 0.05
PR_S1N = vInputArray(5)

PRt = PRc * (1 - dP_cmb) / PR_S1N
tau3 = (T3 + 459.67) / (T1 + 459.67)
ll = LHV / (Cpa * (T1 + 459.67))

w_comp = 1 - PRc ^ (ka / eta_c)
T2 = (T1 + 459.67) * PRc ^ (ka / eta_c) - 459.67

DEN = ll * (T1 + 459.67) + ksi_f * (TF - Tref) - ksi_g * (T3 - Tref)
NUMER = ksi_g * (T3 - Tref) - (T2 - Tref)
f = NUMER / DEN

w_turb = (1 + f) * ksi_g * tau3 * (1 - 1 / PRt ^ (kg * eta_t))
T4 = (T3 + 459.67) / PRt ^ (kg * eta_t) - 459.67

'Set heat balance error to zero:
HB_Err = f * (ll + ksi_f * (TF - Tref) / (T1 + 459.67)) - (w_comp + w_turb) - (1 + f) * ksi_g * (T4 - Tref) / (T1 + 459.67) _
    + (T1 - Tref) / (T1 + 459.67)
HB_Err = HB_Err / (f * ll)

If iOutput = 1 Then
    f = f * c_f
    GTURBUC = eta_m * eta_gen * (w_comp + w_turb) / (f * ll)
ElseIf iOutput = 2 Then
    GTURBUC = eta_m * eta_gen * (w_comp + w_turb)
ElseIf iOutput = 3 Then
    GTURBUC = T4 + T4_Corr
Else
    GTURBUC = HB_Err * 100
End If

Exit Function
err_here:

End Function


Function ARCCOS(alfa As Single, iMode As Integer)
' imode = 1 in rads
' imode = 2 in degs
ARCCOS = ARCTAN(-alfa / Sqr(-alfa * alfa + 1), 1) + 2 * ARCTAN(1, 1)
If iMode = 2 Then
    ARCCOS = (ARCCOS / 3.1416) * 180
End If

End Function
Function ARCTAN(alfa As Single, iMode As Integer) As Single
' imode = 1 in rads
' imode = 2 in degs

ARCTAN = CSng(Atn(CDbl(alfa)))
If iMode = 2 Then
    ARCTAN = (ARCTAN / 3.1416) * 180
End If

End Function

Function SECANT(alfa As Single) As Single
' alfa = in rads
' sec = 1/cos

SECANT = 1 / CSng(Cos(CDbl(alfa)))

End Function

Function I_xx_ovr_y_max(alfa As Single, t_ovr_c As Single)
'Figure 7.16
'Page 274
'Saravanamuttoo et al., Edition 3
'This is the z that goes into the bending stress calculation, Equation 10.45
'
Dim B As Single
Dim n As Single

If (alfa < 40) Then alfa = 40
If (alfa > 120) Then alfa = 120

B = -0.03 * alfa ^ 2 - 3.9701 * alfa + 1166.7
n = -0.0087 * alfa + 2.0385

I_xx_ovr_y_max = (1 / B) * (10 * t_ovr_c) ^ n

End Function

Function Eff_Smith(fi As Single, psi As Single) As Single
' Smith Chart Efficiency Contours
' eff = f(phi, psi)
' Between phi = 0.4 and 0.8
' From the data in R.I. Lewis
' S. C. Gulen
'
Dim e_fi40 As Single
Dim e_fi45 As Single
Dim e_fi50 As Single
Dim e_fi55 As Single
Dim e_fi60 As Single
Dim e_fi65 As Single
Dim e_fi70 As Single
Dim e_fi75 As Single
Dim e_fi80 As Single

' Constant phi lines, eff = f(psi)
e_fi40 = 3.7064 * psi ^ 2 - 16.787 * psi + 107.23
e_fi45 = 3.6849 * psi ^ 2 - 16.618 * psi + 108.03
e_fi50 = 57.73431 + 44.71177 * Exp(-0.1738504 * psi)
e_fi55 = -0.7574 * psi ^ 3 + 3.5221 * psi ^ 2 - 10.844 * psi + 103.68
e_fi60 = 97.83737 - (2.033634 / -0.5241612) * (1 - Exp(0.5241612 * psi))
e_fi65 = 1.2675 * psi ^ 3 - 9.0907 * psi ^ 2 + 15.303 * psi + 86.724
e_fi70 = 85.58303 + (94.09355 - 85.58303) / (1 + (psi / 2.254925) ^ 6.538703)
e_fi75 = -2.2122 * psi ^ 2 + 3.7992 * psi + 92.796
e_fi80 = 85.89201 + 13.34151 * psi - 6.801581 * psi ^ 2 + 0.763089 * psi ^ 3

' Exact phi
If fi <= 0.4 Then
    Eff_Smith = e_fi40
    Exit Function
ElseIf fi = 0.45 Then
    Eff_Smith = e_fi45
    Exit Function
ElseIf fi = 0.5 Then
    Eff_Smith = e_fi50
    Exit Function
ElseIf fi = 0.55 Then
    Eff_Smith = e_fi55
    Exit Function
ElseIf fi = 0.6 Then
    Eff_Smith = e_fi60
    Exit Function
ElseIf fi = 0.65 Then
    Eff_Smith = e_fi65
    Exit Function
ElseIf fi = 0.7 Then
    Eff_Smith = e_fi70
    Exit Function
ElseIf fi = 0.75 Then
    Eff_Smith = e_fi75
    Exit Function
ElseIf fi >= 0.8 Then
    Eff_Smith = e_fi80
    Exit Function
End If

'Interpolated phi
If fi > 0.4 And fi < 0.45 Then
    Eff_Smith = e_fi40 + (fi - 0.4) / 0.05 * (e_fi45 - e_fi40)
ElseIf fi > 0.45 And fi < 0.5 Then
    Eff_Smith = e_fi45 + (fi - 0.45) / 0.05 * (e_fi50 - e_fi45)
ElseIf fi > 0.5 And fi < 0.55 Then
    Eff_Smith = e_fi50 + (fi - 0.5) / 0.05 * (e_fi55 - e_fi50)
ElseIf fi > 0.55 And fi < 0.6 Then
    Eff_Smith = e_fi55 + (fi - 0.55) / 0.05 * (e_fi60 - e_fi55)
ElseIf fi > 0.6 And fi < 0.65 Then
    Eff_Smith = e_fi60 + (fi - 0.6) / 0.05 * (e_fi65 - e_fi60)
ElseIf fi > 0.65 And fi < 0.7 Then
    Eff_Smith = e_fi65 + (fi - 0.65) / 0.05 * (e_fi70 - e_fi65)
ElseIf fi > 0.7 And fi < 0.75 Then
    Eff_Smith = e_fi70 + (fi - 0.7) / 0.05 * (e_fi75 - e_fi70)
ElseIf fi > 0.75 And fi < 0.8 Then
    Eff_Smith = e_fi75 + (fi - 0.75) / 0.05 * (e_fi80 - e_fi75)
End If

End Function

Function Eff_341(fi As Single, psi As Single, DoR As Single, eps As Single, H_to_b As Single) As Single
' Equation 3.41 in R.I. Lewis
' Also in Saravanamuttoo
' eps is deflection degrees
' H_to_b is the blade chord aspect ratio
Dim zeta_S2 As Single
Dim zeta_R3 As Single
Dim AA As Single, BB As Single

' Soderberg loss correlation
' Equation 3.36 in Lewis
zeta_S2 = 0.025 * (1 + (eps / 90) ^ 2)
zeta_S2 = zeta_S2 * (1 + 3.2 / H_to_b)
'
zeta_R3 = zeta_S2

AA = fi ^ 2 + (psi / 2 + 1 - DoR) ^ 2
BB = fi ^ 2 + (psi / 2 + DoR) ^ 2

Eff_341 = 1 / (1 + 0.5 * (AA * zeta_S2 + BB * zeta_R3) / psi)


End Function
Function alfa3(psi As Single, Rxn As Single) As Single

alfa3 = -27 * (psi - 1)
If Rxn = 0.5 Then
    Exit Function
Else
    If Rxn < 0.5 Then
        alfa3 = alfa3 + 5 * (0.5 - Rxn) * 10
    Else
        alfa3 = alfa3 - 10 * (Rxn - 0.5) * 10
    End If
End If

End Function

Function alfa2(psi As Single, Rxn As Single) As Single

alfa2 = 1.5 * psi + 62.75
If Rxn = 0.5 Then
    Exit Function
Else
    If Rxn < 0.5 Then
        alfa2 = alfa2 + 1 * (0.5 - Rxn) * 10
    Else
        alfa2 = alfa2 - 3 * (Rxn - 0.5) * 10
    End If
End If

End Function

Function TURB_STG(vInputArray As Variant) As Variant
'
' Chapter 10 "Turbine Aero"
' Sample stage calculation
' Uses the "uncooled" isentropic efficiency approach
' with reduced stage total pressure ratio (via momentum losses)
' Stator and rotor cooling flows are calculated elsewhere
' Simplified property functions for Gas, i.e., Cpg(Tg), gamma(Tg) and h(Tg)and Tg(hg)
' Constant Cpa for Air
' In SI Units for simplicity
'
Dim rpm As Single
Dim psi As Single
Dim phi As Single
Dim v1 As Single, v2 As Single, v3 As Single
Dim u1 As Single, u2 As Single, u3 As Single
Dim a_1 As Single, a_2 As Single, a_3 As Single
Dim Ma_1 As Single, Ma_2 As Single, Ma_3 As Single, Ma_u As Single
Dim alfa1 As Single, alfa2 As Single, alfa3 As Single
Dim beta1 As Single, beta2 As Single, beta3 As Single
Dim r_m As Single, r_h As Single, r_t As Single, r_m_new As Single
Dim U As Single, Va1 As Single, Va2 As Single, Va3 As Single
Dim Vr1 As Single, Vr2 As Single, Vr3 As Single
Dim mdot As Single, mdot_2 As Single, mdot_3 As Single
Dim T1tot As Single, P1tot As Single, h1tot As Single
Dim T2tot As Single, P2tot As Single, h2tot As Single
Dim T3tot As Single, P3tot As Single, h3tot As Single
Dim T1st As Single, P1st As Single, h1st As Single
Dim T2st As Single, P2st As Single, h2st As Single
Dim T3st As Single, P3st As Single, h3st As Single
Dim mdot_c_st As Single, Ttot_c_st As Single
Dim mdot_c_rot As Single, Ttot_c_rot As Single
Dim Ttot_c_fws As Single, Ttot_c_aws As Single
Dim Cpa As Single, TF As Single, Stage_PR As Single, hmix As Single, hmix_tot_in As Single
Dim P3tot_p As Single, Stage_PR_short As Single, HTR As Single, ht_ratio As Single
Dim gamma As Single, kappa As Single, rho_gas As Single, Err_r As Single
Dim T3tot_s As Single, T13_avg As Single, T23_avg As Single, Deltah_stage As Single
Dim h_rr As Single, T_rr As Single, h_rr_st As Single, T_rr_st As Single
Dim Eff_uc As Single, Eff_uc_new As Single, Numerator As Single, Denominator As Single, tau_32 As Single
Dim Eff_c As Single, Eff_poly As Single
Dim Fwd_ws_purge_frac As Single, Aft_ws_purge_frac As Single
Dim Fwd_ws_momloss_frac As Single, Aft_ws_momloss_frac As Single
Dim mdot_c_fws As Single, mdot_c_aws As Single
Dim h3st_p As Single, T3st_p As Single, h3tot_p As Single, T3tot_p As Single, Tmix_tot_in As Single
Dim Stg_Rxn As Single, omega As Single, Cpg As Single
Dim Rgas As Single, Rair As Single
Dim i As Integer
Const pi As Single = 3.141593
Const Runv As Single = 8314 'J/kmol-K
Const eps As Single = 0.001
Dim vOutputArray(1 To 30, 1) As Single

Cpa = 0.25 * 4190   'J/kg-K
Rair = Runv / 28.856    'J/kg-K
Rgas = Runv / 28.3    'J/kg-K

rpm = vInputArray(1)
psi = vInputArray(2)
phi = vInputArray(3)
alfa3 = vInputArray(4)  'in degs
mdot = vInputArray(5)   'in kg/s
T1tot = vInputArray(6)  'in K
P1tot = vInputArray(7)  'in bar
Stage_PR = vInputArray(8)
Eff_uc = vInputArray(9)
Fwd_ws_purge_frac = vInputArray(10)
Aft_ws_purge_frac = vInputArray(11)

Fwd_ws_momloss_frac = vInputArray(12)
Aft_ws_momloss_frac = vInputArray(13)

mdot_c_st = vInputArray(14) 'in kg/s
Ttot_c_st = vInputArray(15) 'in K

mdot_c_rot = vInputArray(16)    'in kg/s
Ttot_c_rot = vInputArray(17)    'in K

Ttot_c_fws = vInputArray(18)    'in kg/s
Ttot_c_aws = vInputArray(19)    'in K
alfa1 = vInputArray(21)    'in degs

' Equation 10.14 plus 10.16
Stg_Rxn = 1 + phi * Tan(alfa1 / 180 * pi) - psi / 2
' Equations 10.14 - 10.17
' Note: angles are in degrees
' But trigonometric function arguments are in radians!
' Do not forget the conversion...
beta3 = ARCTAN((psi + 2 * Stg_Rxn) / 2 / phi, 2)
beta2 = ARCTAN((psi - 2 * Stg_Rxn) / 2 / phi, 2)
'alfa3 = alfa1    'not necessary (alfa3 is input)
alfa2 = ARCTAN(Tan(beta2 / 180 * pi) + 1 / phi, 2)

r_m = 1 'meter - initial guess
10:
' angular speed:
omega = 2 * pi * rpm / 60
' mean-radius tangential velocity:
U = omega * r_m 'm/s
'
' for subsequent stages, alfa1 = alfa3 of the previous stage
' Vector algebra
u2 = phi * U
v2 = u2 * Tan(alfa2 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
u1 = u2
Va1 = u1 / Cos(alfa1 / 180 * pi)
v1 = u1 * Tan(alfa1 / 180 * pi)
u3 = u2
v2 = u2 * Tan(alfa2 / 180 * pi)
v3 = u3 * Tan(alfa3 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
Va3 = Sqr(u3 ^ 2 + v3 ^ 2)
' check:
alfa3 = ARCCOS(Cos(alfa2 / 180 * pi) * Va2 / Va3, 2)

'Statepoint 1 - Stator Inlet
T1st = T1tot - 100  'initial guess
For i = 1 To 10
    TF = (T1st * 1.8 - 459.67)  'in degs F
    ' Equation 8.5 - used in several places
    Cpg = (0.2243 + 0.0316 * (TF / 1000)) * 4190 'in J/kg-K
    T1st = T1tot - Va1 ^ 2 / 2 / Cpg
Next i
TF = (T1st * 1.8 - 459.67)  'in degs F
' Enthalpy via integration of Equation 8.5
h1st = (0.2243 * (TF - 77) + 0.0316 * ((TF ^ 2 - 77 ^ 2) / 2000)) * 2326 'in J/kg
h1tot = h1st + Va1 ^ 2 / 2
' Specific heat ratio - from Equation 8.5 with Cvg = Cpg - Rgas
gamma = 2.5127 * T1st ^ -0.089
a_1 = Sqr(gamma * Rgas * T1st)   'speed of sound, m/s
Ma_1 = Va1 / a_1

'Statepoint 2 - Rotor Inlet
mdot_c_fws = Fwd_ws_purge_frac * (mdot + mdot_c_st)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h1tot * mdot + mdot_c_st * Cpa * (Ttot_c_st - 298) + mdot_c_fws * Cpa * (Ttot_c_fws - 298)
mdot_2 = (mdot + mdot_c_st + mdot_c_fws)
h2tot = hmix / mdot_2
h2st = h2tot - Va2 ^ 2 / 2
' Inverse of enthalpy function (curve-fit)
T2st = 808.16 * (h2st / 1000000#) + 432.51
TF = (T2st * 1.8 - 459.67)  'in degs F
Cpg = (0.2243 + 0.0316 * (TF / 1000)) * 4190 'in J/kg-K
' Equation 3.6
T2tot = T2st + Va2 ^ 2 / 2 / Cpg

'Rotor-relative enthalpy and temperature
' Equation 10.9
Vr2 = u2 / Cos(beta2 / 180 * pi)
h_rr = h2tot - (Va2 ^ 2 - Vr2 ^ 2) / 2
h_rr_st = h_rr - Vr2 ^ 2 / 2
T_rr_st = 808.16 * (h_rr_st / 1000000#) + 432.51
TF = (T_rr_st * 1.8 - 459.67)  'in degs F
Cpg = (0.2243 + 0.0316 * (TF / 1000)) * 4190 'in J/kg-K
T_rr = T_rr_st + Vr2 ^ 2 / 2 / Cpg
    
gamma = 2.5127 * T2st ^ -0.089
a_2 = Sqr(gamma * Rgas * T2st)   'speed of sound, m/s
Ma_2 = Va2 / a_2
Ma_u = U / a_2
kappa = 1 - 1 / gamma
' Equation 3.11
P2st = P2tot / (1 + 0.5 * (gamma - 1) * Ma_2 ^ 2) ^ (1 / kappa)
' Ideal gas formula
rho_gas = P2st * 100000 / (Rgas * T2st)

' Pressure bookkeeping
P2tot = (1 - Fwd_ws_momloss_frac) * P1tot
P3tot = P1tot / Stage_PR
P3tot_p = (1 + Aft_ws_momloss_frac) * P3tot
Stage_PR_short = P2tot / P3tot_p

20:
' Statepoint 3 - Rotor Exit
' Expansion across the rotor
For i = 1 To 10
    kappa = 1 - 1 / gamma
    T3tot_s = T2tot * (1 / Stage_PR_short) ^ kappa
    T3tot = Eff_uc * (T3tot_s - T2tot) + T2tot
    ' use a rough estimate for static T3
    T23_avg = (T2st + 0.975 * T3tot) / 2
    gamma = 2.5127 * T23_avg ^ -0.089
Next i

T3st = T3tot - 100  'initial guess
For i = 1 To 10
    TF = (T3st * 1.8 - 459.67)  'in degs F
    Cpg = (0.2243 + 0.0316 * (TF / 1000)) * 4190 'in J/kg-K
    T3st = T3tot - Va3 ^ 2 / 2 / Cpg
    h3st = (0.2243 * (TF - 77) + 0.0316 * ((TF ^ 2 - 77 ^ 2) / 2000)) * 2326 'in J/kg
Next i
h3tot = h3st + Va3 ^ 2 / 2

'Recalculate efficiency:
' From Smith chart (phi = 0.7)
Eff_uc_new = Eff_Smith(phi, psi) / 100
' 1 pct pt debit
Eff_uc_new = Eff_uc_new - 0.01
'Equation 10.21 or 10.24
Eff_uc_new = Eff_341(phi, psi, Stg_Rxn, Abs(beta2 + beta3), 10)
Err_r = Abs(Eff_uc_new / Eff_uc - 1)
If (Err_r > eps) Then
    Eff_uc = Eff_uc_new
    GoTo 20
End If

Deltah_stage = h2tot - h3tot
'Recalculate r_m:
' Equation 10.42
r_m_new = Sqr(Deltah_stage / psi) / omega
Err_r = Abs(r_m_new / r_m - 1)
If (Err_r > eps) Then
    r_m = r_m_new
    GoTo 10
End If

'Calculate hub-tip ratio:
' Equation 10.41
HTR = (mdot_2 / rho_gas) / (4 * pi * r_m ^ 2) / u2
ht_ratio = (1 - HTR) / (1 + HTR)

mdot_c_aws = Aft_ws_purge_frac * (mdot_2 + mdot_c_rot)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h3tot * mdot_2 + mdot_c_rot * (Cpa * (Ttot_c_rot - 298)) + mdot_c_aws * (Cpa * (Ttot_c_aws - 298))
mdot_3 = (mdot_2 + mdot_c_rot + mdot_c_aws)
h3tot_p = hmix / mdot_3
h3st_p = h3tot_p - Va3 ^ 2 / 2
T3st_p = 808.16 * (h3st_p / 1000000#) + 432.51
TF = (T3st_p * 1.8 - 459.67)  'in degs F
Cpg = (0.2243 + 0.0316 * (TF / 1000)) * 4190 'in J/kg-K
T3tot_p = T3st_p + Va3 ^ 2 / 2 / Cpg
'
' Calculate "cooled" stage isentropic and polytropic efficiencies
' Calculate first a pseudo inlet state combining all inlet streams:
    hmix_tot_in = h1tot * mdot + mdot_c_st * Cpa * (Ttot_c_st - 298) + mdot_c_fws * Cpa * (Ttot_c_fws - 298)
    hmix_tot_in = hmix_tot_in + mdot_c_rot * (Cpa * (Ttot_c_rot - 298)) + mdot_c_aws * (Cpa * (Ttot_c_aws - 298))
    hmix_tot_in = hmix_tot_in / mdot_3
    Tmix_tot_in = 808.16 * (hmix_tot_in / 1000000#) + 432.51
' Calculate an average gamma
    T13_avg = (T1st + T3st) / 2
    gamma = 2.5127 * T13_avg ^ -0.089
    kappa = 1 - 1 / gamma
' Calculate the isentropic exit temperature
    T3tot_s = Tmix_tot_in * (1 / Stage_PR) ^ kappa
' Calculate the efficiencies
    Eff_c = (T3tot_p - Tmix_tot_in) / (T3tot_s - Tmix_tot_in)
    Eff_poly = Log(T3tot_p / Tmix_tot_in) / Log(1 / Stage_PR) / kappa


vOutputArray(1, 0) = alfa2
vOutputArray(2, 0) = alfa3
vOutputArray(3, 0) = beta2
vOutputArray(4, 0) = beta3
vOutputArray(5, 0) = Va2
vOutputArray(6, 0) = Va3
vOutputArray(7, 0) = Vr2
vOutputArray(8, 0) = T_rr
vOutputArray(9, 0) = T2tot
vOutputArray(10, 0) = mdot_2
vOutputArray(11, 0) = mdot_3
vOutputArray(12, 0) = Deltah_stage
vOutputArray(13, 0) = T2st
vOutputArray(14, 0) = T3st
vOutputArray(15, 0) = T3st_p
vOutputArray(16, 0) = T3tot
vOutputArray(17, 0) = Stage_PR_short
vOutputArray(18, 0) = r_m
vOutputArray(19, 0) = ht_ratio
vOutputArray(20, 0) = Stg_Rxn
vOutputArray(21, 0) = Eff_uc
vOutputArray(22, 0) = Eff_c
vOutputArray(23, 0) = Eff_poly
vOutputArray(24, 0) = Ma_u
vOutputArray(25, 0) = Va1 / Va3

TURB_STG = vOutputArray

Exit Function
err_here:
TURB_STG = Err.Number

End Function

Function TURB_STGCO2(vInputArray As Variant) As Variant
'
' Chapter 10 "Turbine Aero"
' Sample stage calculation
' Uses the "uncooled" isentropic efficiency approach
' with reduced stage total pressure ratio (via momentum losses)
' Stator and rotor cooling flows are calculated elsewhere
' Simplified property functions for Gas, i.e., Cpg(Tg), gamma(Tg) and h(Tg)and Tg(hg)
' Constant Cpa for Air
' In SI Units for simplicity
'
Dim rpm As Single
Dim psi As Single
Dim phi As Single
Dim v1 As Single, v2 As Single, v3 As Single
Dim u1 As Single, u2 As Single, u3 As Single
Dim a_1 As Single, a_2 As Single, a_3 As Single
Dim Ma_1 As Single, Ma_2 As Single, Ma_3 As Single, Ma_u As Single
Dim alfa1 As Single, alfa2 As Single, alfa3 As Single
Dim beta1 As Single, beta2 As Single, beta3 As Single
Dim r_m As Single, r_h As Single, r_t As Single, r_m_new As Single
Dim U As Single, Va1 As Single, Va2 As Single, Va3 As Single
Dim Vr1 As Single, Vr2 As Single, Vr3 As Single
Dim mdot As Single, mdot_2 As Single, mdot_3 As Single
Dim T1tot As Single, P1tot As Single, h1tot As Single
Dim T2tot As Single, P2tot As Single, h2tot As Single
Dim T3tot As Single, P3tot As Single, h3tot As Single
Dim T1st As Single, P1st As Single, h1st As Single
Dim T2st As Single, P2st As Single, h2st As Single
Dim T3st As Single, P3st As Single, h3st As Single
Dim mdot_c_st As Single, Ttot_c_st As Single
Dim mdot_c_rot As Single, Ttot_c_rot As Single
Dim Ttot_c_fws As Single, Ttot_c_aws As Single
Dim Cpa As Single, TF As Single, Stage_PR As Single, hmix As Single, hmix_tot_in As Single
Dim P3tot_p As Single, Stage_PR_short As Single, HTR As Single, ht_ratio As Single
Dim gamma As Single, kappa As Single, rho_gas As Single, rho_gas2 As Single, rho_gas3 As Single, Err_r As Single
Dim T3tot_s As Single, T13_avg As Single, T23_avg As Single, Deltah_stage As Single
Dim h_rr As Single, T_rr As Single, h_rr_st As Single, T_rr_st As Single
Dim Eff_uc As Single, Eff_uc_new As Single, Numerator As Single, Denominator As Single, tau_32 As Single
Dim Eff_c As Single, Eff_poly As Single
Dim h_0 As Single
Dim Fwd_ws_purge_frac As Single, Aft_ws_purge_frac As Single
Dim Fwd_ws_momloss_frac As Single, Aft_ws_momloss_frac As Single
Dim mdot_c_fws As Single, mdot_c_aws As Single
Dim h3st_p As Single, T3st_p As Single, h3tot_p As Single, T3tot_p As Single, Tmix_tot_in As Single
Dim Stg_Rxn As Single, omega As Single, Cpg As Single
Dim Rgas As Single, Rair As Single
Dim i As Integer
Const pi As Single = 3.141593
Const Runv As Single = 8314 'J/kmol-K
Const eps As Single = 0.001
Dim vOutputArray(1 To 30, 1) As Single
On Local Error GoTo err_here

Cpa = 0.25 * 4190   'J/kg-K
Rair = Runv / 28.856    'J/kg-K
' CO2 properties
Rgas = Runv / 44.01    'J/kg-K
Cpg = vInputArray(11) * 4190 'in J/kg-K
h_0 = 177.43

rpm = vInputArray(1)
psi = vInputArray(2)
phi = vInputArray(3)
alfa3 = vInputArray(4)  'in degs
mdot = vInputArray(5)   'in kg/s
T1tot = vInputArray(6)  'in K
P1tot = vInputArray(7)  'in bar
Stage_PR = vInputArray(8)
Eff_uc = vInputArray(9)
' Specific heat ratio
gamma = vInputArray(10)

mdot_c_st = vInputArray(14) 'in kg/s
Ttot_c_st = vInputArray(15) 'in K

mdot_c_rot = vInputArray(16)    'in kg/s
Ttot_c_rot = vInputArray(17)    'in K

Ttot_c_fws = vInputArray(18)    'in kg/s
Ttot_c_aws = vInputArray(19)    'in K

' Equation 10.18 and 10.20
Stg_Rxn = 1 + phi * Tan(alfa3 / 180 * pi) - psi / 2
' Equations 10.18 - 10.21
' Note: angles are in degrees
' But trigonometric function arguments are in radians!
' Do not forget the conversion...
beta3 = ARCTAN((psi + 2 * Stg_Rxn) / 2 / phi, 2)
beta2 = ARCTAN((psi - 2 * Stg_Rxn) / 2 / phi, 2)
' alfa3 = ARCTAN(Tan(beta3 / 180 * pi) - 1 / phi, 2)    !not necessary (alfa3 is input)
alfa2 = ARCTAN(Tan(beta2 / 180 * pi) + 1 / phi, 2)

r_m = vInputArray(13) 'meter - initial guess
10:
' angular speed:
omega = 2 * pi * rpm / 60
' mean-radius tangential velocity:
U = omega * r_m 'm/s
'u1 = vInputArray(20)    'm/s
'
' for subsequent stages, alfa1 = alfa3 of the previous stage
' Vector algebra
u2 = phi * U
v2 = u2 * Tan(alfa2 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
u1 = u2
Va1 = u1 / Cos(alfa1 / 180 * pi)
v1 = u1 * Tan(alfa1 / 180 * pi)
u3 = u2
v2 = u2 * Tan(alfa2 / 180 * pi)
v3 = u3 * Tan(alfa3 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
Va3 = Sqr(u3 ^ 2 + v3 ^ 2)
' check:
alfa3 = ARCCOS(Cos(alfa2 / 180 * pi) * Va2 / Va3, 2)

'Statepoint 1 - Stator Inlet
'T1st = T1tot - 100  'initial guess
'For i = 1 To 10
'    TF = (T1st * 1.8 - 459.67)  'in degs F
    T1st = T1tot - Va1 ^ 2 / 2 / Cpg
'Next i
TF = (T1st * 1.8 - 459.67)  'in degs F
' Enthalpy via curve-fit
h1st = (vInputArray(11) * TF - h_0) * 2326 'in J/kg
h1tot = h1st + Va1 ^ 2 / 2
a_1 = Sqr(gamma * Rgas * T1st)   'speed of sound, m/s
Ma_1 = Va1 / a_1

'Statepoint 2 - Rotor Inlet
mdot_c_fws = Fwd_ws_purge_frac * (mdot + mdot_c_st)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h1tot * mdot + mdot_c_st * Cpa * (Ttot_c_st - 298) + mdot_c_fws * Cpa * (Ttot_c_fws - 298)
mdot_2 = (mdot + mdot_c_st + mdot_c_fws)
h2tot = hmix / mdot_2
h2st = h2tot - Va2 ^ 2 / 2
' Inverse of enthalpy function
TF = (h2st / 2326 + h_0) / vInputArray(11)   'in degs F
T2st = (TF + 459.67) / 1.8  'in degs K
' Equation 3.6
T2tot = T2st + Va2 ^ 2 / 2 / Cpg

'Rotor-relative enthalpy and temperature
' Equation 10.9
Vr2 = u2 / Cos(beta2 / 180 * pi)
h_rr = h2tot - (Va2 ^ 2 - Vr2 ^ 2) / 2
h_rr_st = h_rr - Vr2 ^ 2 / 2
TF = (h_rr_st / 2326 + h_0) / vInputArray(11)   'in degs F
T_rr_st = (TF + 459.67) / 1.8  'in degs K
T_rr = T_rr_st + Vr2 ^ 2 / 2 / Cpg
    
a_2 = Sqr(gamma * Rgas * T2st)   'speed of sound, m/s
Ma_2 = Va2 / a_2
Ma_u = U / a_2
kappa = 1 - 1 / gamma
' Equation 3.11
P2st = P2tot / (1 + 0.5 * (gamma - 1) * Ma_2 ^ 2) ^ (1 / kappa)
' Equation 10.27
rho_gas = P2st * 100000 / (Rgas * T2st)
rho_gas2 = rho_gas

' Pressure bookkeeping
P2tot = (1 - Fwd_ws_momloss_frac) * P1tot
P3tot = P1tot / Stage_PR
P3tot_p = (1 + Aft_ws_momloss_frac) * P3tot
Stage_PR_short = P2tot / P3tot_p

20:
' Statepoint 3 - Rotor Exit
' Expansion across the rotor
For i = 1 To 10
    kappa = 1 - 1 / gamma
    T3tot_s = T2tot * (1 / Stage_PR_short) ^ kappa
    T3tot = Eff_uc * (T3tot_s - T2tot) + T2tot
    ' use a rough estimate for static T3
    T23_avg = (T2st + 0.975 * T3tot) / 2
Next i

T3st = T3tot - 100  'initial guess
For i = 1 To 10
    TF = (T3st * 1.8 - 459.67)  'in degs F
    T3st = T3tot - Va3 ^ 2 / 2 / Cpg
    h3st = (vInputArray(11) * TF - h_0) * 2326 'in J/kg
Next i
h3tot = h3st + Va3 ^ 2 / 2

a_3 = Sqr(gamma * Rgas * T3st)   'speed of sound, m/s
Ma_3 = Va3 / a_2
' Equation 3.11
P3st = P3tot / (1 + 0.5 * (gamma - 1) * Ma_3 ^ 2) ^ (1 / kappa)
' Equation 10.27
rho_gas = P3st * 100000 / (Rgas * T3st)
rho_gas3 = rho_gas

'Recalculate efficiency:
' From Smith chart (phi = 0.7)
Eff_uc_new = Eff_Smith(phi, psi) / 100
'Equation 10.22 or 10.25
Eff_uc_new = Eff_341(phi, psi, Stg_Rxn, Abs(beta2 + beta3), 10)
' Calibration
Eff_uc_new = Eff_uc_new - vInputArray(12)
Err_r = Abs(Eff_uc_new / Eff_uc - 1)
If (Err_r > eps) Then
    Eff_uc = Eff_uc_new
    GoTo 20
End If

mdot_c_aws = Aft_ws_purge_frac * (mdot_2 + mdot_c_rot)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h3tot * mdot + mdot_c_rot * (Cpa * (Ttot_c_rot - 298)) + mdot_c_aws * (Cpa * (Ttot_c_aws - 298))
mdot_3 = (mdot_2 + mdot_c_rot + mdot_c_aws)
h3tot_p = hmix / mdot_3
h3st_p = h3tot_p - Va3 ^ 2 / 2
TF = (h3st_p / 2326 + h_0) / vInputArray(11)   'in degs F
T3st_p = (TF + 459.67) / 1.8    'in degs K
T3tot_p = T3st_p + Va3 ^ 2 / 2 / Cpg

Deltah_stage = h1tot - h3tot_p
'Recalculate r_m:
' Equation 10.39
r_m_new = Sqr(Deltah_stage / psi) / omega
Err_r = Abs(r_m_new / r_m - 1)
If (Err_r > eps) Then
    r_m = r_m_new
    GoTo 10
End If

'Calculate hub-tip ratio:
' Equation 10.28
HTR = (mdot_3 / rho_gas3) / (4 * pi * r_m ^ 2) / (Va2 * Cos(alfa2 / 180 * pi))
ht_ratio = (1 - HTR) / (1 + HTR)

' Calculate the efficiencies
Eff_c = Eff_uc
Eff_poly = Eff_uc

err_here:
vOutputArray(1, 0) = alfa2
vOutputArray(2, 0) = alfa3
vOutputArray(3, 0) = beta2
vOutputArray(4, 0) = beta3
vOutputArray(5, 0) = Va2
vOutputArray(6, 0) = Va3
vOutputArray(7, 0) = Vr2
vOutputArray(8, 0) = T_rr
vOutputArray(9, 0) = T2tot
vOutputArray(10, 0) = mdot_2
vOutputArray(11, 0) = mdot_3
vOutputArray(12, 0) = Deltah_stage
vOutputArray(13, 0) = T2st
vOutputArray(14, 0) = T3st
vOutputArray(15, 0) = T3st_p
vOutputArray(16, 0) = T3tot
vOutputArray(17, 0) = Stage_PR_short
vOutputArray(18, 0) = r_m
vOutputArray(19, 0) = ht_ratio
vOutputArray(20, 0) = Stg_Rxn
vOutputArray(21, 0) = Eff_uc
vOutputArray(22, 0) = Eff_c
vOutputArray(23, 0) = Eff_poly
vOutputArray(24, 0) = Ma_u
vOutputArray(25, 0) = P3st

TURB_STGCO2 = vOutputArray

Exit Function
TURB_STGCO2 = Err.Number

End Function

Function TURB_STGHE(vInputArray As Variant) As Variant
'
' Chapter 10 "Turbine Aero"
' Sample stage calculation
' Uses the "uncooled" isentropic efficiency approach
' with reduced stage total pressure ratio (via momentum losses)
' Stator and rotor cooling flows are calculated elsewhere
' Simplified property functions for Gas, i.e., Cpg(Tg), gamma(Tg) and h(Tg)and Tg(hg)
' Constant Cpa for Air
' In SI Units for simplicity
'
Dim rpm As Single
Dim psi As Single
Dim phi As Single
Dim v1 As Single, v2 As Single, v3 As Single
Dim u1 As Single, u2 As Single, u3 As Single
Dim a_1 As Single, a_2 As Single, a_3 As Single
Dim Ma_1 As Single, Ma_2 As Single, Ma_3 As Single, Ma_u As Single
Dim alfa1 As Single, alfa2 As Single, alfa3 As Single
Dim beta1 As Single, beta2 As Single, beta3 As Single
Dim r_m As Single, r_h As Single, r_t As Single, r_m_new As Single
Dim U As Single, Va1 As Single, Va2 As Single, Va3 As Single
Dim Vr1 As Single, Vr2 As Single, Vr3 As Single
Dim mdot As Single, mdot_2 As Single, mdot_3 As Single
Dim T1tot As Single, P1tot As Single, h1tot As Single
Dim T2tot As Single, P2tot As Single, h2tot As Single
Dim T3tot As Single, P3tot As Single, h3tot As Single
Dim T1st As Single, P1st As Single, h1st As Single
Dim T2st As Single, P2st As Single, h2st As Single
Dim T3st As Single, P3st As Single, h3st As Single
Dim mdot_c_st As Single, Ttot_c_st As Single
Dim mdot_c_rot As Single, Ttot_c_rot As Single
Dim Ttot_c_fws As Single, Ttot_c_aws As Single
Dim Cpa As Single, TF As Single, Stage_PR As Single, hmix As Single, hmix_tot_in As Single
Dim P3tot_p As Single, Stage_PR_short As Single, HTR As Single, ht_ratio As Single
Dim gamma As Single, kappa As Single, rho_gas As Single, rho_gas2 As Single, rho_gas3 As Single, Err_r As Single
Dim T3tot_s As Single, T13_avg As Single, T23_avg As Single, Deltah_stage As Single
Dim h_rr As Single, T_rr As Single, h_rr_st As Single, T_rr_st As Single
Dim Eff_uc As Single, Eff_uc_new As Single, Numerator As Single, Denominator As Single, tau_32 As Single
Dim Eff_c As Single, Eff_poly As Single
Dim h_0 As Single
Dim Fwd_ws_purge_frac As Single, Aft_ws_purge_frac As Single
Dim Fwd_ws_momloss_frac As Single, Aft_ws_momloss_frac As Single
Dim mdot_c_fws As Single, mdot_c_aws As Single
Dim h3st_p As Single, T3st_p As Single, h3tot_p As Single, T3tot_p As Single, Tmix_tot_in As Single
Dim Stg_Rxn As Single, omega As Single, Cpg As Single
Dim Rgas As Single, Rair As Single
Dim i As Integer
Const pi As Single = 3.141593
Const Runv As Single = 8314 'J/kmol-K
Const eps As Single = 0.001
Dim vOutputArray(1 To 30, 1) As Single
On Local Error GoTo err_here

Cpa = 0.25 * 4190   'J/kg-K
Rair = Runv / 28.856    'J/kg-K
' CO2 properties
Rgas = Runv / 4.003    'J/kg-K
' Helium Cp = 1.243 Btu/lb-R in McDonald
Cpg = vInputArray(11) * 4190 'in J/kg-K
h_0 = 100.55

rpm = vInputArray(1)
psi = vInputArray(2)
phi = vInputArray(3)
alfa3 = vInputArray(4)  'in degs
mdot = vInputArray(5)   'in kg/s
T1tot = vInputArray(6)  'in K
P1tot = vInputArray(7)  'in bar
Stage_PR = vInputArray(8)
Eff_uc = vInputArray(9)
' Specific heat ratio
' 1.665 in McDonald for helium
gamma = vInputArray(10)

mdot_c_st = vInputArray(14) 'in kg/s
Ttot_c_st = vInputArray(15) 'in K

mdot_c_rot = vInputArray(16)    'in kg/s
Ttot_c_rot = vInputArray(17)    'in K

Ttot_c_fws = vInputArray(18)    'in kg/s
Ttot_c_aws = vInputArray(19)    'in K

' Equation 10.18 and 10.20
Stg_Rxn = 1 + phi * Tan(alfa3 / 180 * pi) - psi / 2
' Equations 10.18 - 10.21
' Note: angles are in degrees
' But trigonometric function arguments are in radians!
' Do not forget the conversion...
beta3 = ARCTAN((psi + 2 * Stg_Rxn) / 2 / phi, 2)
beta2 = ARCTAN((psi - 2 * Stg_Rxn) / 2 / phi, 2)
' alfa3 = ARCTAN(Tan(beta3 / 180 * pi) - 1 / phi, 2)    !not necessary (alfa3 is input)
alfa2 = ARCTAN(Tan(beta2 / 180 * pi) + 1 / phi, 2)

r_m = vInputArray(13) 'meter - initial guess
10:
' angular speed:
omega = 2 * pi * rpm / 60
' mean-radius tangential velocity:
U = omega * r_m 'm/s
'u1 = vInputArray(20)    'm/s
'
' for subsequent stages, alfa1 = alfa3 of the previous stage
' Vector algebra
u2 = phi * U
v2 = u2 * Tan(alfa2 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
u1 = u2
Va1 = u1 / Cos(alfa1 / 180 * pi)
v1 = u1 * Tan(alfa1 / 180 * pi)
u3 = u2
v2 = u2 * Tan(alfa2 / 180 * pi)
v3 = u3 * Tan(alfa3 / 180 * pi)
Va2 = Sqr(u2 ^ 2 + v2 ^ 2)
Va3 = Sqr(u3 ^ 2 + v3 ^ 2)
' check:
alfa3 = ARCCOS(Cos(alfa2 / 180 * pi) * Va2 / Va3, 2)

'Statepoint 1 - Stator Inlet
'T1st = T1tot - 100  'initial guess
'For i = 1 To 10
'    TF = (T1st * 1.8 - 459.67)  'in degs F
    T1st = T1tot - Va1 ^ 2 / 2 / Cpg
'Next i
TF = (T1st * 1.8 - 459.67)  'in degs F
' Enthalpy via curve-fit
h1st = (vInputArray(11) * TF - h_0) * 2326 'in J/kg
h1tot = h1st + Va1 ^ 2 / 2
a_1 = Sqr(gamma * Rgas * T1st)   'speed of sound, m/s
Ma_1 = Va1 / a_1

'Statepoint 2 - Rotor Inlet
mdot_c_fws = Fwd_ws_purge_frac * (mdot + mdot_c_st)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h1tot * mdot + mdot_c_st * Cpa * (Ttot_c_st - 298) + mdot_c_fws * Cpa * (Ttot_c_fws - 298)
mdot_2 = (mdot + mdot_c_st + mdot_c_fws)
h2tot = hmix / mdot_2
h2st = h2tot - Va2 ^ 2 / 2
' Inverse of enthalpy function
TF = (h2st / 2326 + h_0) / vInputArray(11)   'in degs F
T2st = (TF + 459.67) / 1.8  'in degs K
' Equation 3.6
T2tot = T2st + Va2 ^ 2 / 2 / Cpg

'Rotor-relative enthalpy and temperature
' Equation 10.9
Vr2 = u2 / Cos(beta2 / 180 * pi)
h_rr = h2tot - (Va2 ^ 2 - Vr2 ^ 2) / 2
h_rr_st = h_rr - Vr2 ^ 2 / 2
TF = (h_rr_st / 2326 + h_0) / vInputArray(11)   'in degs F
T_rr_st = (TF + 459.67) / 1.8  'in degs K
T_rr = T_rr_st + Vr2 ^ 2 / 2 / Cpg
    
a_2 = Sqr(gamma * Rgas * T2st)   'speed of sound, m/s
Ma_2 = Va2 / a_2
Ma_u = U / a_2
kappa = 1 - 1 / gamma
' Equation 3.11
P2st = P2tot / (1 + 0.5 * (gamma - 1) * Ma_2 ^ 2) ^ (1 / kappa)
' Equation 10.27
rho_gas = P2st * 100000 / (Rgas * T2st)
rho_gas2 = rho_gas

' Pressure bookkeeping
P2tot = (1 - Fwd_ws_momloss_frac) * P1tot
P3tot = P1tot / Stage_PR
P3tot_p = (1 + Aft_ws_momloss_frac) * P3tot
Stage_PR_short = P2tot / P3tot_p

20:
' Statepoint 3 - Rotor Exit
' Expansion across the rotor
For i = 1 To 10
    kappa = 1 - 1 / gamma
    T3tot_s = T2tot * (1 / Stage_PR_short) ^ kappa
    T3tot = Eff_uc * (T3tot_s - T2tot) + T2tot
    ' use a rough estimate for static T3
    T23_avg = (T2st + 0.975 * T3tot) / 2
Next i

T3st = T3tot - 100  'initial guess
For i = 1 To 10
    TF = (T3st * 1.8 - 459.67)  'in degs F
    T3st = T3tot - Va3 ^ 2 / 2 / Cpg
    h3st = (vInputArray(11) * TF - h_0) * 2326 'in J/kg
Next i
h3tot = h3st + Va3 ^ 2 / 2

a_3 = Sqr(gamma * Rgas * T3st)   'speed of sound, m/s
Ma_3 = Va3 / a_2
' Equation 3.11
P3st = P3tot / (1 + 0.5 * (gamma - 1) * Ma_3 ^ 2) ^ (1 / kappa)
' Equation 10.27
rho_gas = P3st * 100000 / (Rgas * T3st)
rho_gas3 = rho_gas

'Recalculate efficiency:
' From Smith chart (phi = 0.7)
Eff_uc_new = Eff_Smith(phi, psi) / 100
'Equation 10.22 or 10.25
Eff_uc_new = Eff_341(phi, psi, Stg_Rxn, Abs(beta2 + beta3), 10)
' Calibration
Eff_uc_new = Eff_uc_new - vInputArray(12)
Err_r = Abs(Eff_uc_new / Eff_uc - 1)
If (Err_r > eps) Then
    Eff_uc = Eff_uc_new
    GoTo 20
End If

mdot_c_aws = Aft_ws_purge_frac * (mdot_2 + mdot_c_rot)
' Mixing of main gas and cooling air flows
' (Note: on total basis)
hmix = h3tot * mdot + mdot_c_rot * (Cpa * (Ttot_c_rot - 298)) + mdot_c_aws * (Cpa * (Ttot_c_aws - 298))
mdot_3 = (mdot_2 + mdot_c_rot + mdot_c_aws)
h3tot_p = hmix / mdot_3
h3st_p = h3tot_p - Va3 ^ 2 / 2
TF = (h3st_p / 2326 + h_0) / vInputArray(11)   'in degs F
T3st_p = (TF + 459.67) / 1.8    'in degs K
T3tot_p = T3st_p + Va3 ^ 2 / 2 / Cpg

Deltah_stage = h1tot - h3tot_p
'Recalculate r_m:
' Equation 10.39
r_m_new = Sqr(Deltah_stage / psi) / omega
Err_r = Abs(r_m_new / r_m - 1)
If (Err_r > eps) Then
    r_m = r_m_new
    GoTo 10
End If

'Calculate hub-tip ratio:
' Equation 10.28
HTR = (mdot_2 / rho_gas2) / (4 * pi * r_m ^ 2) / (Va2 * Cos(alfa2 / 180 * pi))
ht_ratio = (1 - HTR) / (1 + HTR)

' Calculate the efficiencies
Eff_c = Eff_uc
Eff_poly = Eff_uc

err_here:
vOutputArray(1, 0) = alfa2
vOutputArray(2, 0) = alfa3
vOutputArray(3, 0) = beta2
vOutputArray(4, 0) = beta3
vOutputArray(5, 0) = Va2
vOutputArray(6, 0) = Va3
vOutputArray(7, 0) = Vr2
vOutputArray(8, 0) = T_rr
vOutputArray(9, 0) = T2tot
vOutputArray(10, 0) = mdot_2
vOutputArray(11, 0) = mdot_3
vOutputArray(12, 0) = Deltah_stage
vOutputArray(13, 0) = T2st
vOutputArray(14, 0) = T3st
vOutputArray(15, 0) = T3st_p
vOutputArray(16, 0) = T3tot
vOutputArray(17, 0) = Stage_PR_short
vOutputArray(18, 0) = r_m
vOutputArray(19, 0) = ht_ratio
vOutputArray(20, 0) = Stg_Rxn
vOutputArray(21, 0) = Eff_uc
vOutputArray(22, 0) = Eff_c
vOutputArray(23, 0) = Eff_poly
vOutputArray(24, 0) = Ma_u
vOutputArray(25, 0) = Va1 / Va3

TURB_STGHE = vOutputArray

Exit Function
TURB_STGHE = Err.Number

End Function

Function Num_Stages(vInputArray As Variant) As Variant
Dim c_p As Single
Dim gama As Single
Dim T_in As Single
Dim T_ex As Single
Dim rpm As Single
Dim psi As Single
Dim fi As Single
Dim p_avg As Single
Dim T_avg As Single
Dim Rgas As Single
Dim m_dot As Single
Dim h_t As Single
Dim r_m As Single
Dim N_s As Single
Dim A_m As Single
Dim u_m As Single
Dim sp_snd As Single
Dim Ma_m As Single
Dim sigma As Single
Dim iMode As Integer
Const pi As Single = 3.141593
Dim vOutputArray(1 To 30, 1) As Single
On Local Error GoTo err_here

c_p = vInputArray(1)
gama = vInputArray(2)
T_in = vInputArray(3)
T_ex = vInputArray(4)
rpm = vInputArray(5)
psi = vInputArray(6)
fi = vInputArray(7)
p_avg = vInputArray(8)
T_avg = vInputArray(9)
Rgas = vInputArray(10)
m_dot = vInputArray(11)
h_t = vInputArray(12)
iMode = vInputArray(13)

Select Case iMode
    Case 0
        r_m = 0.912544 * (m_dot / (p_avg * 100000 * fi * rpm * (1 - h_t) / (1 + h_t) / Rgas / T_avg)) ^ (1 / 3)
    Case 1
        r_m = 0.8 * (m_dot / (fi * rpm * (1 - h_t) / (1 + h_t))) ^ (1 / 3)
    Case Else
        r_m = 1
End Select
N_s = 91.189 * c_p * (T_in - T_ex) / psi / (rpm * r_m) ^ 2
A_m = 4 * pi * r_m ^ 2 * (1 - h_t) / (1 + h_t)
u_m = fi * pi * rpm * r_m / 30
sp_snd = Sqr(gama * Rgas * T_avg)
Ma_m = u_m / sp_snd
sigma = (7850 * 0.00194032) * (A_m * 1550) * rpm ^ 2 * pi / 1800 / 1.44 ^ 2 / 10000000# 'ksi

err_here:

vOutputArray(1, 0) = CInt(N_s)
vOutputArray(2, 0) = r_m
vOutputArray(3, 0) = A_m
vOutputArray(4, 0) = u_m
vOutputArray(5, 0) = Ma_m
vOutputArray(6, 0) = A_m * 1550 * rpm ^ 2 / 1000000000# 'AN2
vOutputArray(7, 0) = sigma

Num_Stages = vOutputArray

Exit Function
Num_Stages = Err.Description

End Function

Function INTP1(xa As Variant, ya As Variant, xx As Single, n As Integer) As Single
ReDim X(1 To n) As Single
ReDim Y(1 To n) As Single
Dim i As Integer, j As Integer, jl As Integer, jm As Integer, ju As Integer
Dim Cond1 As Boolean, Cond2 As Boolean

For i = 1 To n
    X(i) = xa(i)
    Y(i) = ya(i)
Next i

jl = 0
ju = n + 1
10:
If (ju - jl > 1) Then
    jm = (ju + jl) / 2
    Cond1 = X(n) > X(1)
    Cond2 = xx > X(jm)
    If (Cond1 And Cond2) Or (Not Cond1 And Not Cond2) Then
        jl = jm
    Else
        ju = jm
    End If
    GoTo 10
End If
j = jl

Select Case j
    Case 0
        yy = (xx - X(2)) / (X(1) - X(2)) * (Y(1) - Y(2)) + Y(2)
    Case n
        yy = (xx - X(n - 1)) / (X(n) - X(n - 1)) * (Y(n) - Y(n - 1)) + Y(n - 1)
    Case Else
        yy = (xx - X(j)) / (X(j + 1) - X(j)) * (Y(j + 1) - Y(j)) + Y(j)
End Select

INTP1 = yy

End Function

Function INTP2(xa As Variant, ya As Variant, za As Variant, xx As Single, yy As Single, _
    n As Integer, m As Integer) As Single
ReDim X(1 To n) As Single
ReDim zy(1 To n) As Single
ReDim Y(1 To m) As Single
ReDim z1(1 To m) As Single
ReDim Z(1 To n, 1 To m) As Single
Dim i As Integer, j As Integer, jl As Integer, jm As Integer, ju As Integer
Dim Cond1 As Boolean, Cond2 As Boolean

For i = 1 To n
    X(i) = xa(i)
    For j = 1 To m
        Y(j) = ya(j)
        Z(i, j) = za(i, j)
    Next j
Next i

For j = 1 To m  'for each column j..
    For i = 1 To n
        zy(i) = Z(i, j) 'make an array of size n ..
    Next i
    z1(j) = INTP1(X, zy, xx, n) 'find the value of zy(n) for given x
Next j

zz = INTP1(Y, z1, yy, m)

INTP2 = zz

End Function



