Attribute VB_Name = "modGasProps"
Option Explicit

Function Cp_IG(T As Single, iComp As Integer) As Single
'
' Cp in Btu/lbmol-R
' T in F
'
Dim TR As Single, cp As Single
Const R As Single = 1.986   'Universal Gas Constant, Btu/lbmol-R

TR = T + 459.67 'Rankines

Select Case iComp
    Case 1  'N2
        cp = 3.675 - 0.000671 * TR + 0.000000717 * TR * TR - 0.000000000108 * TR * TR * TR - 2.15E-14 * TR * TR * TR * TR
    Case 2  'O2
        cp = 3.626 - 0.001043 * TR + 0.000002178 * TR * TR - 0.00000000116 * TR * TR * TR - 2.054E-13 * TR * TR * TR * TR
    Case 3  'H2O
        cp = 4.07 - 0.000616 * TR + 0.000001281 * TR * TR - 0.000000000508 * TR * TR * TR - 7.69E-14 * TR * TR * TR * TR
    Case 4  'CO2
        cp = 2.401 + 0.004583 * TR - 0.000002039 * TR * TR + 0.000000000343 * TR * TR * TR
    Case 5  'Ar
        cp = 3.675 - 0.000671 * TR + 0.000000717 * TR * TR - 0.000000000108 * TR * TR * TR - 2.15E-14 * TR * TR * TR * TR
    Case 6  'SO2
        cp = 0
    Case 10  'Air
        cp = 3.653 - 0.0007428 * TR + 0.000001017 * TR * TR - 0.000000000328 * TR * TR * TR - 2.632E-14 * TR * TR * TR * TR
    Case 7  'CH4
        cp = 3.826 - 0.002211 * TR + 0.00000758 * TR * TR - 0.000000003898 * TR * TR * TR - 6.633E-13 * TR * TR * TR * TR
    
End Select

Cp_IG = cp * R

End Function

Function h_IG(T As Single, iComp As Integer) As Single
'
' h in Btu/lbmol
' T in F
'
Dim TR As Single, h As Single
Const R As Single = 1.986   'Universal Gas Constant, Btu/lbmol-R

TR = T + 459.67 'Rankines

Select Case iComp
    Case 1  'N2
        h = 3.675 * TR - 0.000671 * (TR ^ 2 / 2) + 0.000000717 * (TR ^ 3 / 3) - 0.000000000108 * (TR ^ 4 / 4) - 2.15E-14 * (TR ^ 5 / 5)
    Case 2  'O2
        h = 3.626 * TR - 0.001043 * (TR ^ 2 / 2) + 0.000002178 * (TR ^ 3 / 3) - 0.00000000116 * (TR ^ 4 / 4) - 2.054E-13 * (TR ^ 5 / 5)
    Case 3  'H2O
        h = 4.07 * TR - 0.000616 * (TR ^ 2 / 2) + 0.000001281 * (TR ^ 3 / 3) - 0.000000000508 * (TR ^ 4 / 4) - 7.69E-14 * (TR ^ 5 / 5)
    Case 4  'CO2
        h = 2.401 * TR + 0.004583 * (TR ^ 2 / 2) - 0.000002039 * (TR ^ 3 / 3) + 0.000000000343 * (TR ^ 4 / 4)
    Case 5  'Ar
        h = 3.675 * TR - 0.000671 * (TR ^ 2 / 2) + 0.000000717 * (TR ^ 3 / 3) - 0.000000000108 * (TR ^ 4 / 4) - 2.15E-14 * (TR ^ 5 / 5)
    Case 6  'SO2
        h = 0
    Case 10  'Air
        h = 3.653 * TR - 0.0007428 * (TR ^ 2 / 2) + 0.000001017 * (TR ^ 3 / 3) - 0.000000000328 * (TR ^ 4 / 4) - 2.632E-14 * (TR ^ 5 / 5)
    Case 7  'CH4
        h = 3.826 * TR - 0.002211 * (TR ^ 2 / 2) + 0.00000758 * (TR ^ 3 / 3) - 0.000000003898 * (TR ^ 4 / 4) - 6.633E-13 * (TR ^ 5 / 5)
    
End Select

h_IG = h * R

End Function

Function s_IG(T As Single, p As Single, iComp As Integer) As Single
'
' Cp in Btu/lbmol-R
' T in F
'
Dim TR As Single, s As Single
Const R As Single = 1.986   'Universal Gas Constant, Btu/lbmol-R

TR = T + 459.67 'Rankines
If (p <= 0) Then p = 0.000000001

Select Case iComp
    Case 1  'N2
        s = 3.675 * Log(TR) - 0.000671 * TR + 0.000000717 * (TR ^ 2 / 2) - 0.000000000108 * (TR ^ 3 / 3) - 2.15E-14 * (TR ^ 4 / 4)
    Case 2  'O2
'        s = 3.626 * Log(TR) - 0.001043 * TR + 0.000002178 * (TR ^ 2 / 2) - 0.00000000116 * (TR ^ 3 / 3) - 2.054E-13 * (TR ^ 4 / 4)
        s = 1.3 * Log(TR) + 16.636  'SCG 11/17/15 to extend abov 1800R
    Case 3  'H2O
'        s = 4.07 * Log(TR) - 0.000616 * TR + 0.000001281 * (TR ^ 2 / 2) - 0.000000000508 * (TR ^ 3 / 3) - 7.69E-14 * (TR ^ 4 / 4)
        s = 1.7683 * Log(TR) - 0.1743   'SCG 11/17/15 to extend abov 1800R
    Case 4  'CO2
        s = 2.401 * Log(TR) + 0.004583 * TR - 0.000002039 * (TR ^ 2 / 2) + 0.000000000343 * (TR ^ 3 / 3)
    Case 5  'Ar
        s = 3.675 * Log(TR) - 0.000671 * TR + 0.000000717 * (TR ^ 2 / 2) - 0.000000000108 * (TR ^ 3 / 3) - 2.15E-14 * (TR ^ 4 / 4)
    Case 6  'SO2
        s = 0
    Case 10  'Air
        s = 3.653 * Log(TR) - 0.0007428 * TR + 0.000001017 * (TR ^ 2 / 2) - 0.000000000328 * (TR ^ 3 / 3) - 2.632E-14 * (TR ^ 4 / 4)
    Case 7  'CH4
        s = 3.826 * Log(TR) - 0.002211 * TR + 0.00000758 * (TR ^ 2 / 2) - 0.000000003898 * (TR ^ 3 / 3) - 6.633E-13 * (TR ^ 4 / 4)
    
End Select

s_IG = s * R
s_IG = s_IG - R * Log(p)

End Function

Function MW(iComp As Integer) As Single
'
' Molecular Weight, lb/lbmol
' from data in PTC 4.4

Select Case iComp
    Case 1  'N2
        MW = 28.01348
    Case 2  'O2
        MW = 31.9988
    Case 3  'H2O
        MW = 18.01528
    Case 4  'CO2
        MW = 44.098
    Case 5  'Ar
        MW = 39.948
    Case 6  'SO2
        MW = 64.0648
    Case 10  'Air
        MW = 28.97
    Case 7  'CH4
        MW = 16.04
    
End Select

End Function

Function hgas(T As Single, Comp As Variant, iNumComp As Integer, Optional iMode As Integer) As Single
'
Dim i As Integer
Dim h As Single
Dim Y(1 To 10) As Single
Dim X(1 To 10) As Single

If IsMissing(iMode) Then iMode = 1
h = 0
For i = 1 To iNumComp
    If (iMode = 1) Then
        Y(i) = Comp(i)  'mole fractions..
        h = h + Y(i) * h_IG(T, i)   'Btu/mole
    Else
' PTC 4.4, Sheet I calculation (see page 79)
        X(i) = Comp(i)  'mass fractions
        h = h + X(i) * h_i_PTC4(T, i) 'Btu / lb
    End If
Next i
hgas = h

End Function

Function MWgas(Comp As Variant, iNumComp As Integer) As Single
'
Dim i As Integer
Dim m As Single
Dim Y(1 To 10) As Single

m = 0
For i = 1 To iNumComp
    Y(i) = Comp(i)
    m = m + Y(i) * MW(i)
Next i
MWgas = m

End Function

Function Cpgas(T As Single, Comp As Variant, iNumComp As Integer, Optional iMode As Integer) As Single
'
Dim i As Integer
Dim C As Single, mwt As Single, h1 As Single, h2 As Single
Dim Y(1 To 10) As Single
Dim X(1 To 10) As Single

If IsMissing(iMode) Then iMode = 1

C = 0
Select Case iMode
    Case 1
        For i = 1 To iNumComp
            Y(i) = Comp(i)
            C = C + Y(i) * Cp_IG(T, i)
        Next i
    Case 2
        mwt = MWgas(Comp, iNumComp)
        For i = 1 To iNumComp
            X(i) = (1 / mwt) * Comp(i) * MW(i)
        Next i
        h2 = hgas(T + 0.1, X, 6, 2)
        h1 = hgas(T - 0.1, X, 6, 2)
        C = (h2 - h1) / 0.2
        C = C * mwt
End Select
Cpgas = C
'Err.Description
End Function

Function gamma(T As Single, Comp As Variant, iNumComp As Integer)
Const R As Single = 1.986   'Universal Gas Constant, Btu/lbmol-R
Dim cp As Single, cv As Single

cp = Cpgas(T, Comp, iNumComp, 2)
cv = cp - R
gamma = cp / cv

End Function

Function sgas(T As Single, p As Single, Comp As Variant, iNumComp As Integer) As Single
'
Dim i As Integer
Dim s As Single, p_i As Single
Dim Y(1 To 10) As Single

s = 0
For i = 1 To iNumComp
    Y(i) = Comp(i)  'mole fraction of component i
    p_i = Y(i) * p  'partial pressure of component i
    s = s + Y(i) * s_IG(T, p_i, i)
Next i
sgas = s

End Function

Function Psat(T As Single) As Single
'
Dim TC As Single, p As Single

Psat = Pvap(T)
'Exit Function

TC = (T - 32) / 1.8

If (TC < 100) Then
    p = 10 ^ (8.07131 - 1730.63 / (233.426 + TC)) 'in mm Hg
Else
    p = 10 ^ (8.14019 - 1810.94 / (244.485 + TC))
End If

Psat = p * (14.7 / 760.09)

End Function

Function Pvap(T As Single) As Single
'
' PTC 4.4, 5-2.1.3
' T in F
'
Dim C1 As Double, C2 As Double, C3 As Double, C4 As Double, C5 As Double, C6 As Double, C7 As Double
Dim dLnPv As Double, TR As Double

If T < -148 Then T = -148
If T > 392 Then T = 392
TR = CDbl(T + 459.67)

If T < 32 Then
    C1 = -10214.165
    C2 = -4.8932428
    C3 = -0.0053765794
    C4 = 0.00000019202377
    C5 = 3.5575832E-10
    C6 = -9.0344688E-14
    C7 = 4.1635019
Else
    C1 = -10440.397
    C2 = -11.29465
    C3 = -0.027022355
    C4 = 0.00001289036
    C5 = -2.4780681E-09
    C6 = 0#
    C7 = 6.5459673
End If

dLnPv = C1 / TR + C2 + C3 * TR + C4 * (TR ^ 2) + C5 * (TR ^ 3) + C6 * (TR ^ 4) + C7 * Log(TR)

Pvap = CSng(Exp(dLnPv))

End Function

Function ExhGasComp(Mair As Single, MWAIR As Single, nfuel As Single, CompAir As Variant, iNumComp As Integer) As Variant
'
' mair = Mass flow rate of air
' MWair = Molecular weight of air
' yair() = Mole fraction composition of air into GT
' nfuel = Number of moles of CH4
'
' only for pure CH4 fuel !!!
'
Dim nair As Single, nN2 As Single, nO2 As Single, nH2O As Single, nCO2 As Single, nExh As Single, nAr As Single
Dim nSO2 As Single
Dim yAir(1 To 10) As Single
Dim yexh(1 To 10) As Single
Dim Temp(1 To 6, 1) As Variant

Dim i As Integer

nair = Mair / MWAIR
nExh = nair + nfuel

For i = 1 To iNumComp
    yAir(i) = CompAir(i)  'mole fraction of component i
Next i

' Moles of N2+Ar, O2, H2O and CO2 coming in with air
nN2 = yAir(1) * nair
nO2 = yAir(2) * nair
nH2O = yAir(3) * nair
nCO2 = yAir(4) * nair
nAr = yAir(5) * nair
nSO2 = yAir(6) * nair

' Moles of N2+Ar, O2, H2O and CO2 in exhaust
nN2 = nN2
nAr = nAr
nSO2 = nSO2
nCO2 = nCO2 + nfuel
nH2O = nH2O + 2 * nfuel
nO2 = nO2 - 2 * nfuel

yexh(1) = nN2 / nExh
yexh(2) = nO2 / nExh
yexh(3) = nH2O / nExh
yexh(4) = nCO2 / nExh
yexh(5) = nAr / nExh
yexh(6) = nSO2 / nExh
For i = 1 To iNumComp
    Temp(i, 1) = yexh(i) 'mole fraction of component i
Next i

ExhGasComp = Temp

End Function

Function h_i_PTC4(T As Single, iComp As Integer) As Single
'
Dim TR As Double
Dim a1 As Double, a2 As Double, a3 As Double, a4 As Double, a5 As Double, a6 As Double
Dim a7 As Double, a8 As Double
Dim dEnth As Double
'
TR = CDbl(T + 459.67)
'5-2.6.3 Enthalpy Equation Constants. The enthalpy is calculated for each constituent in a gas stream
'as a function of the gas temperature. The enthalpy correlations for each constituent are dual range correlations
'TR = Temperature [R] = (Temperature [F] + 459.67)
'
If (TR <= 1800#) Then
    Select Case iComp
        Case 1  'N2
            a1 = 5076.903444
            a2 = -48.72474552
            a3 = 0.431208942
            a4 = -16.79893331
            a5 = 100.9860003
            a6 = -2925.146767
            a7 = 34031.29176
            a8 = -123.5670205
        Case 2  'O2
            a1 = -6888.073538
            a2 = 54.14599817
            a3 = 0.069447256
            a4 = 7.402341031
            a5 = -4.364913417
            a6 = -538.2934954
            a7 = 12285.5047
            a8 = 406.9617332
        Case 3  'H2O
            a1 = -14100.39482
            a2 = 114.2053432
            a3 = 0.102713623
            a4 = 22.11623913
            a5 = -83.27114025
            a6 = 2341.441933
            a7 = -28077.80325
            a8 = 844.3000544
        Case 4  'CO2
            a1 = 7227.711078
            a2 = -50.87920389
            a3 = 0.239235281
            a4 = 3.138394549
            a5 = -0.987579956
            a6 = -148.749738
            a7 = 2449.873977
            a8 = -199.50475
        Case 5  'Ar
            a1 = 0
            a2 = 0
            a3 = 0.124279476
            a4 = 0
            a5 = 0
            a6 = 0
            a7 = 0
            a8 = 64.58431507
        Case 6  'SO2
            a1 = -5333.964877
            a2 = 50.72160967
            a3 = -0.073060273
            a4 = 18.98189113
            a5 = -80.07268636
            a6 = 1921.86454
            a7 = -19897.19608
            a8 = 330.8196608
    End Select

Else

    Select Case iComp
        Case 1  'N2
            a1 = 134989.0343
            a2 = -285.7350587
            a3 = 0.43008964
            a4 = -1.209016552
            a5 = 1.088015054
            a6 = -5.844054022
            a7 = 14.34281709
            a8 = -1809.588277
        Case 2  'O2
            a1 = -208707.2821
            a2 = 261.9417179
            a3 = 0.112934906
            a4 = 2.185673559
            a5 = -1.397060777
            a6 = 5.463669086
            a7 = -9.687873053
            a8 = 2037.040157
        Case 3  'CO2
            a1 = 17207.41433
            a2 = -145.2915082
            a3 = 0.374147054
            a4 = -0.115607258
            a5 = 0.022579091
            a6 = -0.365792061
            a7 = 5.44194572
            a8 = -758.4598037
        Case 4  'H2O
            a1 = 369646.9091
            a2 = -478.7281864
            a3 = 0.512156854
            a4 = 7.018191588
            a5 = -7.753574689
            a6 = 44.54356391
            a7 = -101.277945
            a8 = -3313.267173
        Case 5  'Ar
            a1 = 3.238297928
            a2 = -0.005362306
            a3 = 0.124282926
            a4 = -0.0000551268
            a5 = 0.0000616422
            a6 = -0.000387631
            a7 = 0.001021528
            a8 = 64.54706733
        Case 6  'SO2
            a1 = -11325.49719
            a2 = -46.04550381
            a3 = 0.236090695
            a4 = -0.172156415
            a5 = 0.180366793
            a6 = -0.724777306
            a7 = 1.723497991
            a8 = -151.4826348
    End Select

End If
' A1 A2 A3 A4*105 A5*109 A6*1014 A7*1019 A8
a4 = a4 * 0.00001
a5 = a5 * 0.000000001
a6 = a6 * 0.00000000000001
a7 = a7 * 1E-19
'
'5-2.6.1 General. The gas stream enthalpy is a mass weighted value of the stream constituent
'enthalpies. The constituent enthalpy equations are derived from NASA correlations (Ref. NASA Reference
'Publication 2002-211556, September 2002).
'
dEnth = -a1 / TR + a2 * Log(TR) + a3 * TR + a4 * TR ^ 2 + a5 * TR ^ 3 + a6 * TR ^ 4 + a7 * TR ^ 5 - a8
h_i_PTC4 = CSng(dEnth)

End Function

Function h_CO2_Real(T As Single, p As Single) As Single
'
' T in F
' p in psia
'
Dim TK As Single, pMPa As Single, h As Single

TK = (T - 32) / 1.8 + 273.15
pMPa = (p / 1000000) / 0.00014504
' h in kJ/kg
' h = 1.24842440091074 * TK + 50916.0541685375 / (TK + 23.8130024523638 * PMpa) - 111.585229639213

'h = h_CO2_Table(pMPa, TK)

h = 175.827849738172 + 0.757763667650031 * TK + 0.109657394430063 * TK * pMPa + 1.45076024929479E-02 * pMPa ^ 2 + _
2.5526784472071E-04 * TK ^ 2 + 5.0827664538335E-08 * TK ^ 3 * pMPa - 32.6790111909589 * pMPa - 1.28535969030694E-04 * TK ^ 2 * pMPa

h_CO2_Real = h * 0.42992    'Btu/lb

End Function

Function s_CO2_Real(T As Single, p As Single) As Single
'
' T in F
' p in psia
'
Dim TK As Single, pMPa As Single, s As Single

TK = (T - 32) / 1.8 + 273.15
pMPa = (p / 1000000) / 0.00014504
' s in kJ/kg-K
' s = 4.85567151888651 / (PMpa + Sqr(Sqr(TK))) + Sqr(Sqr(TK)) - 3.31648086143795

' s = s_CO2_Table(PMpa, TK)

s = 1.00877743444577 + 3.57253319411314E-03 * TK + 3.62930003623897E-02 * pMPa ^ 2 + 1.39949910324164E-05 * TK * pMPa + _
6.67314177662569E-05 * pMPa ^ 4 - 0.237237730409643 * pMPa - 1.35986024270316E-06 * TK ^ 2 - 2.68414087443878E-03 * pMPa ^ 3

s_CO2_Real = s * 0.238846    'Btu/lb-R

End Function

Function h_CO2_Table(p As Single, T As Single)
'
' p in MPa
' T in K
'
Dim h_1000 As Single, h_900 As Single, h_800 As Single, h_700 As Single
Dim h_600 As Single, h_500 As Single, h_400 As Single, h_300 As Single
Dim hi As Single, lo As Single, btw As Single

' Isotherm enthalpies as f(p)
h_1000 = 0.00580679 * p ^ 2 - 0.552792 * p + 1187.15
h_900 = 0.00718553 * p ^ 2 - 0.794375 * p + 1065.24
h_800 = 0.00886375 * p ^ 2 - 1.1129 * p + 946.442
h_700 = 0.0108844 * p ^ 2 - 1.55626 * p + 831.442
h_600 = 0.0127427 * p ^ 2 - 2.21319 * p + 721.106
h_500 = 0.0114075 * p ^ 2 - 3.29022 * p + 616.534
h_400 = 0.00198679 * p ^ 3 - 0.0833087 * p ^ 2 - 4.90794 * p + 518.618
If (p <= 10) Then
    h_300 = -0.211579 * p ^ 3 + 0.7918 * p ^ 2 - 11.4343 * p + 430.804
Else
    h_300 = -1.4143 * p + 198.2
End If

' Cap at 1000 K (max) and 300 K (min)
If (T > 1000) Then T = 1000
If (T < 300) Then T = 300

' Interpolate
If (T <= 1000 And T > 900) Then
    hi = h_1000
    lo = h_900
    btw = (T - 900) / 100 * (hi - lo) + lo
ElseIf (T <= 900 And T > 800) Then
    hi = h_900
    lo = h_800
    btw = (T - 800) / 100 * (hi - lo) + lo
ElseIf (T <= 800 And T > 700) Then
    hi = h_800
    lo = h_700
    btw = (T - 700) / 100 * (hi - lo) + lo
ElseIf (T <= 700 And T > 600) Then
    hi = h_700
    lo = h_600
    btw = (T - 600) / 100 * (hi - lo) + lo
ElseIf (T <= 600 And T > 500) Then
    hi = h_600
    lo = h_500
    btw = (T - 500) / 100 * (hi - lo) + lo
ElseIf (T <= 500 And T > 400) Then
    hi = h_500
    lo = h_400
    btw = (T - 400) / 100 * (hi - lo) + lo
ElseIf (T <= 400 And T >= 300) Then
    hi = h_400
    lo = h_300
    btw = (T - 300) / 100 * (hi - lo) + lo
Else
    btw = -999
End If

h_CO2_Table = btw

End Function

Function T_CO2_Real(h As Single, p As Single) As Single
'
' h in Btu/lb
' p in psia
'
Dim TK As Single, pMPa As Single, hSI As Single

pMPa = (p / 1000000) / 0.00014504
hSI = h / 0.42992
' h in kJ/kg
TK = 22.1616892866994 * pMPa + 1.15336786489034 * hSI + 5.38222756773835E-05 * pMPa * hSI ^ 2 - 154.105349969687 - _
1.54728809950197E-04 * hSI ^ 2 - 4.63864805161158E-02 * pMPa ^ 2 - 5.69814718181892E-02 * pMPa * hSI - 1.73833440380097E-08 * pMPa * hSI ^ 3
'h = h_CO2_Table(PMpa, TK)

T_CO2_Real = (TK - 273.15) * 1.8 + 32 'F

End Function

Function T_CO2_Real_PS(s As Single, p As Single) As Single
'
' s in Btu/lb-R
' p in psia
'
Dim TK As Single, pMPa As Single, sSI As Single

pMPa = (p / 1000000) / 0.00014504
sSI = s / 0.238846
' h in kJ/kg
TK = 357.703669410751 + 194.394738425952 * sSI ^ 2 + 55.0385590954222 * pMPa * sSI + 0.684680898366365 * pMPa ^ 3 * sSI _
- 427.236356790222 * sSI - 8.98180136130603 * pMPa ^ 2 * sSI - 1.73783073120788E-02 * pMPa ^ 4 * sSI
'h = h_CO2_Table(PMpa, TK)

T_CO2_Real_PS = (TK - 273.15) * 1.8 + 32 'F

End Function

Function s_CO2_Table(p As Single, T As Single)
'
' p in MPa
' T in K
'
Dim s_1000 As Single, s_900 As Single, s_800 As Single, s_700 As Single
Dim s_600 As Single, s_500 As Single, s_400 As Single, s_300 As Single
Dim hi As Single, lo As Single, btw As Single

' Isotherm enthalpies as f(p)
s_1000 = 3.06144 * p ^ -0.070399
s_900 = 2.93391 * p ^ -0.0744448
s_800 = 2.7952 * p ^ -0.0795652
s_700 = 2.64362 * p ^ -0.0864436
s_600 = -0.206306 * Log(p) + 2.46432
s_500 = -0.218458 * Log(p) + 2.277
s_400 = -0.00025953 * p ^ 3 + 0.00973393 * p ^ 2 - 0.135532 * p + 2.15653
If (p <= 10) Then
    s_300 = -0.0019544 * p ^ 3 + 0.0311852 * p ^ 2 - 0.236175 * p + 1.98064
Else
    s_300 = -0.0225 * p + 1.0081
End If

' Cap at 1000 K (max) and 300 K (min)
If (T > 1000) Then T = 1000
If (T < 300) Then T = 300

' Interpolate
If (T <= 1000 And T > 900) Then
    hi = s_1000
    lo = s_900
    btw = (T - 900) / 100 * (hi - lo) + lo
ElseIf (T <= 900 And T > 800) Then
    hi = s_900
    lo = s_800
    btw = (T - 800) / 100 * (hi - lo) + lo
ElseIf (T <= 800 And T > 700) Then
    hi = s_800
    lo = s_700
    btw = (T - 700) / 100 * (hi - lo) + lo
ElseIf (T <= 700 And T > 600) Then
    hi = s_700
    lo = s_600
    btw = (T - 600) / 100 * (hi - lo) + lo
ElseIf (T <= 600 And T > 500) Then
    hi = s_600
    lo = s_500
    btw = (T - 500) / 100 * (hi - lo) + lo
ElseIf (T <= 500 And T > 400) Then
    hi = s_500
    lo = s_400
    btw = (T - 400) / 100 * (hi - lo) + lo
ElseIf (T <= 400 And T >= 300) Then
    hi = s_400
    lo = s_300
    btw = (T - 300) / 100 * (hi - lo) + lo
Else
    btw = -999
End If

s_CO2_Table = btw

End Function

Function Den_CO2_Real(p As Single, T As Single)
'
Dim DEN As Double, Pnorm As Double, Tnorm As Double
Dim pMPa As Single, TK As Single, V As Single

pMPa = (p / 1000000) / 0.00014504
TK = (T - 32) / 1.8 + 273.15

Pnorm = p / 1000
Tnorm = T / 1000

DEN = 3.74013804024777 * Pnorm + Pnorm / Tnorm + 0.810599929442191 ^ Pnorm / (4.76460496627105E-03 + 452.517471614216 * Tnorm ^ 4) - _
(0.810599929442191 ^ Pnorm / (4.76460496627105E-03 + 452.517471614216 * Tnorm ^ 4) + _
Sqr(3.74013804024777 * Pnorm)) ^ (1.55382590837741 * Tnorm)

' sp. vol. in cubic meters per kg
V = (1.94225254236194E-02 * (TK / 100) - 4.14034867158911E-03) / pMPa

'v = -1.90124306409093E-02 / (TK / 100) ^ Sqr(TK / 100) + _
'(1.90124306409093E-02 * TK / 100 - 6.83908856054667E-04) / pMPa

DEN = (1 / V) / 16.018

Den_CO2_Real = CSng(DEN)

End Function

Function s_CO2_ProMax(p As Single, T As Single) As Single
' p in psia
' T in F
Dim s As Single

p = p / 1000
T = T / 1000

s = 0.397515274523544 + 7.80867138668049 * T + 76.6428268891021 * T ^ 3 + 25.6891304478178 * T ^ 5 _
- 3.11008785382665E-02 * p - 36.0595042022849 * T ^ 2 - 73.2875128548434 * T ^ 4

s_CO2_ProMax = s

End Function

Function h_CO2_ProMax(p As Double, T As Double) As Double
' p in psia
' T in F
Dim h As Double

p = p / 1000
T = T / 1000

h = 4192.09712922859 * T + 35284.5231999062 * T ^ 3 + 4984.47522038217 * T ^ 6 - 4172.55034929241 _
- 12.6552732709828 * p - 18282.3563318519 * T ^ 2 - 25636.5065875082 * T ^ 4

h_CO2_ProMax = h

End Function

Function rtbis(x1 As Single, x2 As Single, xacc As Single) As Single
Const JMAX As Integer = 40
Dim j As Integer
Dim dx As Single, f As Single, fmid As Single, xmid As Single
fmid = x2
f = x1
If (f * fmid >= 0) Then
End If
If (f < 0) Then
    rtbis = x1
    dx = x2 - x1
Else
    rtbis = x2
    dx = x1 - x2
End If
For j = 1 To JMAX
    dx = dx / 2
    xmid = rtbis + dx
    fmid = xmid
    If (fmid <= 0) Then
        rtbis = xmid
    End If
    If (Abs(dx) < xacc Or fmid = 0) Then
        Exit Function
    End If
Next j
End Function

Function Find_Tgas(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
h_o As Single) As Single
' Find gas temperature as a function of enthalpy & composition
' h_o is the enthalpy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.001
Dim j As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
'
1:
fmid = h_o - hgas(T2, Comp, iNumComp, iMode)
f = h_o - hgas(T1, Comp, iNumComp, iMode)
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        Find_Tgas = -999
        Exit Function
    End If
    T2 = T2 + 10
    If (T2 > 4000) Then
        Find_Tgas = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    fmid = h_o - hgas(Tmid, Comp, iNumComp, iMode)
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        Find_Tgas = rtbis
        Exit Function
    End If
Next j
Find_Tgas = -999
End Function

Function h_Fuel(sTemp As Single, iFuel As Integer, iUnits As Integer) As Single
Dim T As Single, TK As Single
Dim Af(1 To 8) As Single, MWf As Single
'
' if iUnits =1 (British), sTemp should be passed in degrees F, sPres in psi
' Otherwise (SI), sTemp should be passed in degrees C, sPres in bar
' Convert to degrees Kelvin and atmospheres here:
If iUnits = 1 Then
    TK = (sTemp - 32) / 1.8 + 273.15
Else
    TK = sTemp + 273.15
End If

T = TK / 1000

Select Case iFuel
    Case 1  'Methane
        Af(1) = -0.29149
        Af(2) = 26.327
        Af(3) = -10.61
        Af(4) = 1.5656
        Af(5) = 0.16573
        Af(6) = -18.331
        Af(7) = 0
        Af(8) = 4.3
        MWf = 16.04
    Case 2
    Case 3
    Case 4
    Case Else
End Select
    
h_Fuel = Af(1) * T + Af(2) * T * T / 2 + Af(3) * T * T * T / 3 + Af(4) * T * T * T * T / 4 - _
    Af(5) / T + Af(6) + Af(8)   'kcal/gmol
h_Fuel = h_Fuel / MWf 'kcal/g
If iUnits <> 1 Then
    h_Fuel = h_Fuel * 4186.8 'kJ/kg
Else
    h_Fuel = h_Fuel * 1800 'Btu/lb
End If
End Function


