Attribute VB_Name = "modJANAF"
Option Explicit
Dim a1(0 To 1, 1 To 7) As Single   'CO2
Dim a2(0 To 1, 1 To 7) As Single   'H2O
Dim a3(0 To 1, 1 To 7) As Single   'CO
Dim a4(0 To 1, 1 To 7) As Single   'H2
Dim a5(0 To 1, 1 To 7) As Single   'O2
Dim a6(0 To 1, 1 To 7) As Single   'N2
Dim a7(0 To 1, 1 To 7) As Single   'OH
Dim a8(0 To 1, 1 To 7) As Single   'NO
Dim a9(0 To 1, 1 To 7) As Single   'O
Dim a10(0 To 1, 1 To 7) As Single  'H
Dim n_u As Single, n_fuel As Single, n_b As Single
Dim n_O2 As Single
Dim n_N2 As Single
Dim n_CO2 As Single
Dim n_H2O As Single
Dim n_CO As Single
Dim n_H2 As Single
Dim phi As Single, MW_Fuel As Single, Y As Single, eps As Single, psi As Single
Dim EGR As Single, x_b As Single, x_r As Single, x_c As Single
Dim m_RP As Single, MW_b As Single, MW_u As Single, m_air As Single
Const Tburned As Single = 1000

Function props(strProp As String, sTemp As Single, sPres As Single, vComp As Variant, iUnits As Integer) As Single
Dim Y(1 To 10) As Single
Dim i As Integer
Dim sngProp As Single, sngMW As Single
Dim Temp As Single, TK As Single, PATM As Single
Dim sngSumMoleFracs As Single
On Local Error GoTo err_here
' sTemp = Temperature
' sPres = Pressure
' vComp = Composition Array
'
' if iUnits =1 (British), sTemp should be passed in degrees F, sPres in psi
' Otherwise (SI), sTemp should be passed in degrees C, sPres in bar
' Convert to degrees Kelvin and atmospheres here:
If iUnits = 1 Then
    TK = (sTemp - 32) / 1.8 + 273.15
    PATM = sPres / 14.696
Else
    TK = sTemp + 273.15
    PATM = sPres / 1.01325
End If
' Species List:
'1   CO2
'2   H2O
'3   CO
'4   H2
'5   O2
'6   N2 (Include AR here..)
'7   OH
'8   NO
'9   O
'10  H
'
sngSumMoleFracs = 0
For i = 1 To 10
    Y(i) = vComp(i)
    If (Y(i) < 0) Then Y(i) = 0
    If (Y(i) > 1) Then Y(i) = 1
    sngSumMoleFracs = sngSumMoleFracs + Y(i)
Next i

'Sum of mole fractions should be 1.0...
If sngSumMoleFracs > 1 Then
' ..if not, normalize:
    For i = 1 To 10
        Y(i) = Y(i) / sngSumMoleFracs
    Next i
End If

sngProp = 0
' Calculate mixture properties per unit mole:
Call Get_Coefs  'get the coefficients (Table 4.10 in Heywood)
' Equation 4.39 for specific heat, cp
' Equation 4.40 for enthalpy, h
' Equation 4.41 for entropy, s
' From section 4.7 in
' J. B. Heywood, "Internal Combustion Engine Fundamentals," 1988, McGraw-Hill
'
Select Case Trim(strProp)
    Case "cp", "Cp", "CP", "cP" 'Specific heat
        For i = 1 To 10
            sngProp = sngProp + Y(i) * Cp_JANAF(i, TK, iUnits)
        Next i
    Case "h", "H"   'Enthalpy
        For i = 1 To 10
            sngProp = sngProp + Y(i) * h_JANAF(i, TK, iUnits)
        Next i
    Case "s", "S"   'Entropy
        For i = 1 To 10
            sngProp = sngProp + Y(i) * s_JANAF(i, TK, iUnits)
        Next i
    Case "mw", "MW"
        For i = 1 To 10
            sngProp = sngProp + Y(i) * MW_JANAF(i)
        Next i
        props = sngProp
        Exit Function
End Select

' Calculate mixture molecular weight:
sngMW = 0
For i = 1 To 10
    sngMW = sngMW + Y(i) * MW_JANAF(i)
Next i
' Convert to per unit mass basis:
' c_p in Btu/lb-R or kJ/kg-K
' h in Btu/lb or kJ/kg
' s in Btu/lb-R or kJ/kg-K
Select Case Trim(strProp)
    Case "s", "S"
    ' Pressure dependency of entropy:
        If iUnits = 1 Then
            props = (sngProp - 1.986 * Log(PATM)) / sngMW
        Else
            props = (sngProp - 8.314 * Log(PATM)) / sngMW
        End If
    Case Else
        props = sngProp / sngMW
End Select
Exit Function
err_here:
    props = -999
End Function

Function Cp_JANAF(iSpecies As Integer, TK As Single, iUnits As Integer) As Single
Dim Cp_over_R As Single
Dim i As Integer
Dim j As Integer
' TK is temperature in degrees Kelvin
' From section 4.7 in
' J. B. Heywood, "Internal Combustion Engine Fundamentals," 1988, McGraw-Hill
If TK <= Tburned Then
' Tburned = 1000K = 1340F
    i = 0
Else
    i = 1
End If

Cp_over_R = 0
' Calculate c_p / R (non-dimensional)
' Equation 4.39, p. 130
For j = 1 To 5
    Select Case iSpecies
        Case 1
            Cp_over_R = Cp_over_R + a1(i, j) * TK ^ (j - 1)
        Case 2
            Cp_over_R = Cp_over_R + a2(i, j) * TK ^ (j - 1)
        Case 3
            Cp_over_R = Cp_over_R + a3(i, j) * TK ^ (j - 1)
        Case 4
            Cp_over_R = Cp_over_R + a4(i, j) * TK ^ (j - 1)
        Case 5
            Cp_over_R = Cp_over_R + a5(i, j) * TK ^ (j - 1)
        Case 6
            Cp_over_R = Cp_over_R + a6(i, j) * TK ^ (j - 1)
        Case 7
            Cp_over_R = Cp_over_R + a7(i, j) * TK ^ (j - 1)
        Case 8
            Cp_over_R = Cp_over_R + a8(i, j) * TK ^ (j - 1)
        Case 9
            Cp_over_R = Cp_over_R + a9(i, j) * TK ^ (j - 1)
        Case 10
            Cp_over_R = Cp_over_R + a10(i, j) * TK ^ (j - 1)
    End Select
Next j
'
' c_p in British or SI units (per unit mole)
If iUnits = 1 Then
    Cp_JANAF = Cp_over_R * 1.986 'British in Btu/lbmol-R
Else
    Cp_JANAF = Cp_over_R * 8.314 'SI in kJ/kmol-K
End If
      
End Function

Function h_JANAF(iSpecies As Integer, TK As Single, iUnits As Integer) As Single
Dim h_over_RT As Single, TR As Single
Dim i As Integer
Dim j As Integer
' TK is temperature in degrees Kelvin
' From section 4.7 in
' J. B. Heywood, "Internal Combustion Engine Fundamentals," 1988, McGraw-Hill
'
If TK <= Tburned Then
' Tburned = 1000K = 1340F
    i = 0
Else
    i = 1
End If

h_over_RT = 0
' Calculate h / RT (non-dimensional)
' Equation 4.40, p. 130
For j = 1 To 6
    Select Case iSpecies
        Case 1
            If j <= 5 Then
                h_over_RT = h_over_RT + a1(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a1(i, j) / TK
            End If
        Case 2
            If j <= 5 Then
                h_over_RT = h_over_RT + a2(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a2(i, j) / TK
            End If
        Case 3
            If j <= 5 Then
                h_over_RT = h_over_RT + a3(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a3(i, j) / TK
            End If
        Case 4
            If j <= 5 Then
                h_over_RT = h_over_RT + a4(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a4(i, j) / TK
            End If
        Case 5
            If j <= 5 Then
                h_over_RT = h_over_RT + a5(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a5(i, j) / TK
            End If
        Case 6
            If j <= 5 Then
                h_over_RT = h_over_RT + a6(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a6(i, j) / TK
            End If
        Case 7
            If j <= 5 Then
                h_over_RT = h_over_RT + a7(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a7(i, j) / TK
            End If
        Case 8
            If j <= 5 Then
                h_over_RT = h_over_RT + a8(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a8(i, j) / TK
            End If
        Case 9
            If j <= 5 Then
                h_over_RT = h_over_RT + a9(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a9(i, j) / TK
            End If
        Case 10
            If j <= 5 Then
                h_over_RT = h_over_RT + a10(i, j) * TK ^ (j - 1) / j
            Else
                h_over_RT = h_over_RT + a10(i, j) / TK
            End If
    End Select
Next j
'
' h in British or SI units (per unit mole)
If iUnits = 1 Then
    TR = TK * 1.8
    h_JANAF = h_over_RT * 1.986 * TR 'British in Btu/lbmol
Else
    h_JANAF = h_over_RT * 8.314 * TK 'SI in kJ/kmol
End If
      
End Function

Function s_JANAF(iSpecies As Integer, TK As Single, iUnits As Integer) As Single
Dim s_over_R As Single
Dim i As Integer
Dim j As Integer
' TK is temperature in degrees Kelvin
' From section 4.7 in
' J. B. Heywood, "Internal Combustion Engine Fundamentals," 1988, McGraw-Hill
'
If TK <= Tburned Then
' Tburned = 1000K = 1340F
    i = 0
Else
    i = 1
End If

s_over_R = 0
' Calculate s / R (non-dimensional)
' Equation 4.41, p. 130
For j = 1 To 7
    Select Case iSpecies
        Case 1
            If j = 1 Then
                s_over_R = s_over_R + a1(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a1(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a1(i, j)
            End If
        Case 2
            If j = 1 Then
                s_over_R = s_over_R + a2(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a2(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a2(i, j)
            End If
        Case 3
            If j = 1 Then
                s_over_R = s_over_R + a3(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a3(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a3(i, j)
            End If
        Case 4
            If j = 1 Then
                s_over_R = s_over_R + a4(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a4(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a4(i, j)
            End If
        Case 5
            If j = 1 Then
                s_over_R = s_over_R + a5(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a5(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a5(i, j)
            End If
        Case 6
            If j = 1 Then
                s_over_R = s_over_R + a6(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a6(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a6(i, j)
            End If
        Case 7
            If j = 1 Then
                s_over_R = s_over_R + a7(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a7(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a7(i, j)
            End If
        Case 8
            If j = 1 Then
                s_over_R = s_over_R + a8(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a8(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a8(i, j)
            End If
        Case 9
            If j = 1 Then
                s_over_R = s_over_R + a9(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a9(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a9(i, j)
            End If
        Case 10
            If j = 1 Then
                s_over_R = s_over_R + a10(i, j) * Log(TK)
            ElseIf j <= 5 Then
                s_over_R = s_over_R + a10(i, j) * TK ^ (j - 1) / (j - 1)
            ElseIf j = 6 Then
                'nothing
            Else
                s_over_R = s_over_R + a10(i, j)
            End If
    End Select
Next j
'
' s in British or SI units (per unit mole)
If iUnits = 1 Then
    s_JANAF = s_over_R * 1.986 'British in Btu/lbmol-R
Else
    s_JANAF = s_over_R * 8.314 'SI in kJ/kmol-K
End If
      
End Function

Sub Get_Coefs()
    Dim i As Integer
    Dim j As Integer
'   Table 4.10 in Heywood
    i = 0 'T range 300-1000K - Unburned
    a1(i, 1) = 2.4008
    a1(i, 2) = 0.0087351
    a1(i, 3) = -0.0000066071
    a1(i, 4) = 0.0000000020022
    a1(i, 5) = 6.3274E-16
    a1(i, 6) = -48378#
    a1(i, 7) = 9.6951
    
    a2(i, 1) = 4.0701
    a2(i, 2) = -0.0011084
    a2(i, 3) = 0.0000041521
    a2(i, 4) = -0.0000000029637
    a2(i, 5) = 8.0702E-13
    a2(i, 6) = -30280#
    a2(i, 7) = -0.3227
    
    a3(i, 1) = 3.7101
    a3(i, 2) = -0.0016191
    a3(i, 3) = 0.0000036924
    a3(i, 4) = -0.000000002032
    a3(i, 5) = 2.3953E-13
    a3(i, 6) = -14356#
    a3(i, 7) = 2.9555
    
    a4(i, 1) = 3.0574
    a4(i, 2) = 0.0026765
    a4(i, 3) = -0.0000058099
    a4(i, 4) = 0.000000005521
    a4(i, 5) = -1.8123E-12
    a4(i, 6) = -988.9
    a4(i, 7) = -2.2997
    
    a5(i, 1) = 3.6256
    a5(i, 2) = -0.0018782
    a5(i, 3) = 0.0000070555
    a5(i, 4) = -0.0000000067635
    a5(i, 5) = 2.1556E-12
    a5(i, 6) = -1047.5
    a5(i, 7) = 4.3053
    
    a6(i, 1) = 3.6748
    a6(i, 2) = -0.0012082
    a6(i, 3) = 0.000002324
    a6(i, 4) = -0.00000000063218
    a6(i, 5) = -2.2577E-13
    a6(i, 6) = -1061.2
    a6(i, 7) = 2.358
    
    For j = 1 To 7
        a7(i, j) = 0
        a8(i, j) = 0
        a9(i, j) = 0
        a10(i, j) = 0
    Next j
    
    i = 1 'T range 1000-5000K - Burned
    
    a1(i, 1) = 4.4608
    a1(i, 2) = 0.0030982
    a1(i, 3) = -0.0000012393
    a1(i, 4) = 0.00000000022741
    a1(i, 5) = -1.5526E-14
    a1(i, 6) = -48961#
    a1(i, 7) = -0.098636
    
    a2(i, 1) = 2.7168
    a2(i, 2) = 0.0029451
    a2(i, 3) = -0.00000080224
    a2(i, 4) = 0.00000000010227
    a2(i, 5) = -4.8472E-15
    a2(i, 6) = -29906#
    a2(i, 7) = 6.6306
    
    a3(i, 1) = 2.9841
    a3(i, 2) = 0.0014891
    a3(i, 3) = -0.000000579
    a3(i, 4) = 0.00000000010365
    a3(i, 5) = -6.9354E-15
    a3(i, 6) = -14245#
    a3(i, 7) = 6.3479
    
    a4(i, 1) = 3.1002
    a4(i, 2) = 0.00051119
    a4(i, 3) = 0.000000052644
    a4(i, 4) = -0.00000000003491
    a4(i, 5) = 3.6945E-15
    a4(i, 6) = -877.38
    a4(i, 7) = -1.9629
    
    a5(i, 1) = 3.622
    a5(i, 2) = 0.00073618
    a5(i, 3) = -0.00000019652
    a5(i, 4) = 0.000000000036202
    a5(i, 5) = -2.8946E-15
    a5(i, 6) = -1202#
    a5(i, 7) = 3.6151
    
    a6(i, 1) = 2.8963
    a6(i, 2) = 0.0015155
    a6(i, 3) = -0.00000057235
    a6(i, 4) = 0.000000000099807
    a6(i, 5) = -6.5224E-15
    a6(i, 6) = -905.86
    a6(i, 7) = 6.1615
    
    a7(i, 1) = 2.9106
    a7(i, 2) = 0.00095932
    a7(i, 3) = -0.00000019442
    a7(i, 4) = 0.000000000013757
    a7(i, 5) = 1.4225E-16
    a7(i, 6) = 3935.4
    a7(i, 7) = 5.4423
    
    a8(i, 1) = 3.189
    a8(i, 2) = 0.0013382
    a8(i, 3) = -0.00000052899
    a8(i, 4) = 0.000000000095919
    a8(i, 5) = -6.4848E-15
    a8(i, 6) = 9828.3
    a8(i, 7) = 6.7458
    
    a9(i, 1) = 2.5421
    a9(i, 2) = -0.000027551
    a9(i, 3) = -0.0000000031028
    a9(i, 4) = 4.5511E-12
    a9(i, 5) = -4.3681E-16
    a9(i, 6) = 29231
    a9(i, 7) = 4.9203
    
    a10(i, 1) = 2.5
    a10(i, 2) = 0
    a10(i, 3) = 0
    a10(i, 4) = 0
    a10(i, 5) = 0
    a10(i, 6) = 25472
    a10(i, 7) = -0.46012
    
End Sub

Function MW_JANAF(iComp As Integer) As Single
'
' Molecular Weight, lb/lbmol
' from data in PTC 4.4
Dim MW As Single

Select Case iComp
    Case 1  'CO2
        MW = 44.098
    Case 2  'H2O
        MW = 18.01528
    Case 3  'CO
        MW = 28.0986
    Case 4  'H2
        MW = 2.01588
    Case 5  'O2
        MW = 31.9988
    Case 6  'N2
        MW = 28.01348
    Case 7  'OH
        MW = 17.00734
    Case 8  'NO
        MW = 30.00614
    Case 9  'O
        MW = 15.9994
    Case 10  'H
        MW = 1.00794
   
End Select

MW_JANAF = MW
End Function

Sub Mixture_Comp(phi As Single, TK As Single, x_b As Single)
'
' Tables 4.3 and 4.4 in Heywood
'
Dim k As Single
Dim sArg As Single
Dim AA As Single, BB As Single, CC As Single, C As Single

sArg = 2.743 - 1761 / TK - 1611000# / TK ^ 2 + 280300000# / TK ^ 3  'Eq. (4.5)
k = Exp(sArg)
' Eq. (4.6)
AA = 1
BB = -(k * (2 * (phi - 1) + eps * phi) + 2 * (1 - eps * phi)) / (k - 1)
CC = 2 * k * eps * phi * (phi - 1) / (k - 1)
C = (-BB + Sqr(BB * BB - 4 * AA * CC)) / (2 * AA)

n_fuel = 4 * (1 - x_b) * (1 + 2 * eps) * phi / MW_Fuel
If (phi <= 1) Then
    n_O2 = 1 - x_b * phi
    n_N2 = psi
    n_CO2 = x_b * eps * phi
    n_H2O = 2 * x_b * (1 - eps) * phi
    n_CO = 0
    n_H2 = 0
Else
    n_O2 = 1 - x_b
    n_N2 = psi
    n_CO2 = x_b * (eps * phi - C)
    n_H2O = x_b * (2 * (1 - eps * phi) + C)
    n_CO = x_b * C
    n_H2 = x_b * (2 * (phi - 1) - C)
End If

' x_b = 1 --> burned, otherwise unburned
If (x_b = 1) Then
    n_b = n_O2 + n_N2 + n_CO2 + n_H2O + n_CO + n_H2 + n_fuel
Else
    n_u = n_O2 + n_N2 + n_CO2 + n_H2O + n_CO + n_H2 + n_fuel
End If

End Sub

Function TSV(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
v_o As Single, s_o As Single) As Single
' Find gas temperature as a function of specific volume and entropy
' s_o is the entropy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.001
Dim j As Integer, i As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
Const Runv As Single = 1.986
Dim sngMW As Single, Pres As Single
'
sngMW = 0
For i = 1 To iNumComp
    sngMW = sngMW + Comp(i) * MW_JANAF(i)
Next i
1:
Pres = (Runv / sngMW) * (T2 + 459.67) / v_o * (778.17 / 144)
fmid = s_o - props("s", T2, Pres, Comp, iMode)
Pres = (Runv / sngMW) * (T1 + 459.67) / v_o * (778.17 / 144)
f = s_o - props("s", T1, Pres, Comp, iMode)
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TSV = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TSV = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    Pres = (Runv / sngMW) * (Tmid + 459.67) / v_o * (778.17 / 144)
    fmid = s_o - props("s", Tmid, Pres, Comp, iMode)
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TSV = rtbis
        Exit Function
    End If
Next j
TSV = -999
End Function

Function TSP(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
P_o As Single, s_o As Single) As Single
' Find gas temperature as a function of pressure and entropy
' s_o is the entropy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.001
Dim j As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
'
1:
fmid = s_o - props("s", T2, P_o, Comp, iMode)
f = s_o - props("s", T1, P_o, Comp, iMode)
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TSP = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TSP = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    fmid = s_o - props("s", Tmid, P_o, Comp, iMode)
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TSP = rtbis
        Exit Function
    End If
Next j
TSP = -999
End Function

Function TUV(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
v_o As Single, u_o As Single) As Single
' Find gas temperature as a function of sp. volume and internal energy
' u_o is the internal energy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.001
Dim j As Integer, i As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
Const Runv As Single = 1.986
Dim sngMW As Single, P_o As Single
'
sngMW = 0
For i = 1 To iNumComp
    sngMW = sngMW + Comp(i) * MW_JANAF(i)
Next i
'
1:
P_o = (Runv / sngMW) * (T2 + 459.67) / v_o * (778.17 / 144)
fmid = u_o - (props("h", T2, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
P_o = (Runv / sngMW) * (T1 + 459.67) / v_o * (778.17 / 144)
f = u_o - (props("h", T1, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TUV = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TUV = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    P_o = (Runv / sngMW) * (Tmid + 459.67) / v_o * (778.17 / 144)
    fmid = u_o - (props("h", Tmid, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TUV = rtbis
        Exit Function
    End If
Next j
TUV = -999
End Function

Function TUP(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
P_o As Single, u_o As Single) As Single
' Find gas temperature as a function of pressure and internal energy
' u_o is the internal energy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.001
Dim j As Integer, i As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
Const Runv As Single = 1.986
Dim sngMW As Single, v_o As Single
'
sngMW = 0
For i = 1 To iNumComp
    sngMW = sngMW + Comp(i) * MW_JANAF(i)
Next i
'
1:
v_o = (Runv / sngMW) * (T2 + 459.67) / P_o * (778.17 / 144)
fmid = u_o - (props("h", T2, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
v_o = (Runv / sngMW) * (T1 + 459.67) / P_o * (778.17 / 144)
f = u_o - (props("h", T1, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TUP = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TUP = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    v_o = (Runv / sngMW) * (Tmid + 459.67) / P_o * (778.17 / 144)
    fmid = u_o - (props("h", Tmid, 15, Comp, iMode) - P_o * v_o * 144 / 778.17)
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TUP = rtbis
        Exit Function
    End If
Next j
TUP = -999
End Function

Function TU(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
u_o As Single) As Single
' Find temperature from internal energy
' u_o is the internal energy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.0001
Dim j As Integer, i As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
Const Runv As Single = 1.986    'Btu/lbmol-R
Dim sngMW As Single
'
sngMW = 0
For i = 1 To iNumComp
    sngMW = sngMW + Comp(i) * MW_JANAF(i)
Next i
'
1:
' f = (h - RT) - u
fmid = (props("h", T2, 15, Comp, iMode) - (Runv / sngMW) * (T2 + 459.67)) - u_o
f = (props("h", T1, 15, Comp, iMode) - (Runv / sngMW) * (T1 + 459.67)) - u_o
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TU = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TU = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    fmid = (props("h", Tmid, 15, Comp, iMode) - (Runv / sngMW) * (Tmid + 459.67)) - u_o
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TU = rtbis
        Exit Function
    End If
Next j
TU = -999
End Function

Function TH(T1 As Single, T2 As Single, Comp As Variant, iNumComp As Integer, iMode As Integer, _
h_o As Single) As Single
' Find temperature from enthalpy
' h_o is the enthalpy in question
' Comp is composition with iNumComp components
' T1 and T2 are low and high estimates
' xacc is the desired accuracy
Const JMAX As Integer = 40
Const Tacc As Single = 0.0001
Dim j As Integer, i As Integer
Dim dT As Single, f As Single, fmid As Single, Tmid As Single, rtbis As Single
Const Runv As Single = 1.986    'Btu/lbmol-R
Dim sngMW As Single
'
sngMW = 0
For i = 1 To iNumComp
    sngMW = sngMW + Comp(i) * MW_JANAF(i)
Next i
'
1:
' f = (h - RT) - u
fmid = props("h", T2, 15, Comp, iMode) - h_o
f = props("h", T1, 15, Comp, iMode) - h_o
'
If (f * fmid >= 0) Then
' Root is not bracketed..
    T1 = T1 - 10
    If (T1 < 0) Then
        TH = -999
        Exit Function
    End If
    T2 = T2 + 100
    If (T2 > (5000 * 1.8 - 32)) Then
        TH = -999
        Exit Function
    End If
    GoTo 1
End If
If (f < 0) Then
    rtbis = T1
    dT = T2 - T1
Else
    rtbis = T2
    dT = T1 - T2
End If
For j = 1 To JMAX
    dT = dT / 2
    Tmid = rtbis + dT
    fmid = props("h", Tmid, 15, Comp, iMode) - h_o
    If (fmid <= 0) Then
        rtbis = Tmid
    End If
    If (Abs(dT) < Tacc Or fmid = 0) Then
        TH = rtbis
        Exit Function
    End If
Next j
TH = -999
End Function

